@@ -30,6 +30,7 @@ import org.apache.flume.Event;
 import org.apache.flume.annotations.InterfaceAudience;
 import org.apache.flume.annotations.InterfaceStability;
 import org.apache.flume.channel.file.encryption.KeyProvider;
+import org.apache.flume.channel.file.instrumentation.FileChannelCounter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -63,7 +64,7 @@ import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;
  * Stores FlumeEvents on disk and pointers to the events in a in memory queue.
  * Once a log object is created the replay method should be called to reconcile
  * the on disk write ahead log with the last checkpoint of the queue.
- *
+ * <p>
  * Before calling any of commitPut/commitTake/get/put/rollback/take
  * {@linkplain org.apache.flume.channel.file.Log#lockShared()}
  * should be called. After
@@ -123,10 +124,12 @@ public class Log {
   private boolean didFastReplay = false;
   private boolean didFullReplayDueToBadCheckpointException = false;
   private final boolean useDualCheckpoints;
+  private final boolean compressBackupCheckpoint;
   private volatile boolean backupRestored = false;
 
   private final boolean fsyncPerTransaction;
   private final int fsyncInterval;
+  private final boolean checkpointOnClose;
 
   private int readCount;
   private int putCount;
@@ -136,6 +139,8 @@ public class Log {
 
   private final List<File> pendingDeletes = Lists.newArrayList();
 
+  private final FileChannelCounter channelCounter;
+
   static class Builder {
     private long bCheckpointInterval;
     private long bMinimumRequiredSpace;
@@ -151,11 +156,16 @@ public class Log {
     private String bEncryptionCipherProvider;
     private long bUsableSpaceRefreshInterval = 15L * 1000L;
     private boolean bUseDualCheckpoints = false;
+    private boolean bCompressBackupCheckpoint = false;
     private File bBackupCheckpointDir = null;
 
     private boolean fsyncPerTransaction = true;
     private int fsyncInterval;
 
+    private boolean checkpointOnClose = true;
+
+    private FileChannelCounter channelCounter;
+
     boolean isFsyncPerTransaction() {
       return fsyncPerTransaction;
     }
@@ -212,12 +222,12 @@ public class Log {
       return this;
     }
 
-    Builder setUseLogReplayV1(boolean useLogReplayV1){
+    Builder setUseLogReplayV1(boolean useLogReplayV1) {
       this.useLogReplayV1 = useLogReplayV1;
       return this;
     }
 
-    Builder setUseFastReplay(boolean useFastReplay){
+    Builder setUseFastReplay(boolean useFastReplay) {
       this.useFastReplay = useFastReplay;
       return this;
     }
@@ -242,52 +252,70 @@ public class Log {
       return this;
     }
 
+    Builder setCompressBackupCheckpoint(boolean compressBackupCheckpoint) {
+      this.bCompressBackupCheckpoint = compressBackupCheckpoint;
+      return this;
+    }
+
     Builder setBackupCheckpointDir(File backupCheckpointDir) {
       this.bBackupCheckpointDir = backupCheckpointDir;
       return this;
     }
 
+    Builder setCheckpointOnClose(boolean enableCheckpointOnClose) {
+      this.checkpointOnClose = enableCheckpointOnClose;
+      return this;
+    }
+
+    Builder setChannelCounter(FileChannelCounter channelCounter) {
+      this.channelCounter = channelCounter;
+      return this;
+    }
+
     Log build() throws IOException {
       return new Log(bCheckpointInterval, bMaxFileSize, bQueueCapacity,
-        bUseDualCheckpoints, bCheckpointDir, bBackupCheckpointDir, bName,
-        useLogReplayV1, useFastReplay, bMinimumRequiredSpace,
-        bEncryptionKeyProvider, bEncryptionKeyAlias,
-        bEncryptionCipherProvider, bUsableSpaceRefreshInterval,
-        fsyncPerTransaction, fsyncInterval, bLogDirs);
+          bUseDualCheckpoints, bCompressBackupCheckpoint, bCheckpointDir,
+          bBackupCheckpointDir, bName, useLogReplayV1, useFastReplay,
+          bMinimumRequiredSpace, bEncryptionKeyProvider, bEncryptionKeyAlias,
+          bEncryptionCipherProvider, bUsableSpaceRefreshInterval,
+          fsyncPerTransaction, fsyncInterval, checkpointOnClose, channelCounter, bLogDirs);
     }
   }
 
   private Log(long checkpointInterval, long maxFileSize, int queueCapacity,
-    boolean useDualCheckpoints, File checkpointDir, File backupCheckpointDir,
-    String name, boolean useLogReplayV1, boolean useFastReplay,
-    long minimumRequiredSpace, @Nullable KeyProvider encryptionKeyProvider,
-    @Nullable String encryptionKeyAlias,
-    @Nullable String encryptionCipherProvider,
-    long usableSpaceRefreshInterval, boolean fsyncPerTransaction,
-    int fsyncInterval, File... logDirs)
-          throws IOException {
+              boolean useDualCheckpoints, boolean compressBackupCheckpoint,
+              File checkpointDir, File backupCheckpointDir,
+              String name, boolean useLogReplayV1, boolean useFastReplay,
+              long minimumRequiredSpace, @Nullable KeyProvider encryptionKeyProvider,
+              @Nullable String encryptionKeyAlias,
+              @Nullable String encryptionCipherProvider,
+              long usableSpaceRefreshInterval, boolean fsyncPerTransaction,
+              int fsyncInterval, boolean checkpointOnClose, FileChannelCounter channelCounter,
+              File... logDirs)
+      throws IOException {
     Preconditions.checkArgument(checkpointInterval > 0,
-      "checkpointInterval <= 0");
+        "checkpointInterval <= 0");
     Preconditions.checkArgument(queueCapacity > 0, "queueCapacity <= 0");
     Preconditions.checkArgument(maxFileSize > 0, "maxFileSize <= 0");
     Preconditions.checkNotNull(checkpointDir, "checkpointDir");
     Preconditions.checkArgument(usableSpaceRefreshInterval > 0,
         "usableSpaceRefreshInterval <= 0");
     Preconditions.checkArgument(
-      checkpointDir.isDirectory() || checkpointDir.mkdirs(), "CheckpointDir "
-      + checkpointDir + " could not be created");
+        checkpointDir.isDirectory() || checkpointDir.mkdirs(), "CheckpointDir "
+            + checkpointDir + " could not be created");
     if (useDualCheckpoints) {
       Preconditions.checkNotNull(backupCheckpointDir, "backupCheckpointDir is" +
-        " null while dual checkpointing is enabled.");
+          " null while dual checkpointing is enabled.");
       Preconditions.checkArgument(
-        backupCheckpointDir.isDirectory() || backupCheckpointDir.mkdirs(),
-        "Backup CheckpointDir " + backupCheckpointDir +
-          " could not be created");
+          backupCheckpointDir.isDirectory() || backupCheckpointDir.mkdirs(),
+          "Backup CheckpointDir " + backupCheckpointDir +
+              " could not be created");
     }
     Preconditions.checkNotNull(logDirs, "logDirs");
     Preconditions.checkArgument(logDirs.length > 0, "logDirs empty");
     Preconditions.checkArgument(name != null && !name.trim().isEmpty(),
-            "channel name should be specified");
+        "channel name should be specified");
+    Preconditions.checkNotNull(channelCounter, "ChannelCounter must be not null");
 
     this.channelNameDescriptor = "[channel=" + name + "]";
     this.useLogReplayV1 = useLogReplayV1;
@@ -301,20 +329,20 @@ public class Log {
     locks = Maps.newHashMap();
     try {
       lock(checkpointDir);
-      if(useDualCheckpoints) {
+      if (useDualCheckpoints) {
         lock(backupCheckpointDir);
       }
       for (File logDir : logDirs) {
         lock(logDir);
       }
-    } catch(IOException e) {
+    } catch (IOException e) {
       unlock(checkpointDir);
       for (File logDir : logDirs) {
         unlock(logDir);
       }
       throw e;
     }
-    if(encryptionKeyProvider != null && encryptionKeyAlias != null &&
+    if (encryptionKeyProvider != null && encryptionKeyAlias != null &&
         encryptionCipherProvider != null) {
       LOGGER.info("Encryption is enabled with encryptionKeyProvider = " +
           encryptionKeyProvider + ", encryptionKeyAlias = " + encryptionKeyAlias
@@ -330,7 +358,7 @@ public class Log {
       throw new IllegalArgumentException("Encryption configuration must all " +
           "null or all not null: encryptionKeyProvider = " +
           encryptionKeyProvider + ", encryptionKeyAlias = " +
-          encryptionKeyAlias +  ", encryptionCipherProvider = " +
+          encryptionKeyAlias + ", encryptionCipherProvider = " +
           encryptionCipherProvider);
     }
     open = false;
@@ -338,14 +366,18 @@ public class Log {
     this.maxFileSize = maxFileSize;
     this.queueCapacity = queueCapacity;
     this.useDualCheckpoints = useDualCheckpoints;
+    this.compressBackupCheckpoint = compressBackupCheckpoint;
     this.checkpointDir = checkpointDir;
     this.backupCheckpointDir = backupCheckpointDir;
     this.logDirs = logDirs;
     this.fsyncPerTransaction = fsyncPerTransaction;
     this.fsyncInterval = fsyncInterval;
+    this.checkpointOnClose = checkpointOnClose;
+    this.channelCounter = channelCounter;
+
     logFiles = new AtomicReferenceArray<LogFile.Writer>(this.logDirs.length);
     workerExecutor = Executors.newSingleThreadScheduledExecutor(new
-      ThreadFactoryBuilder().setNameFormat("Log-BackgroundWorker-" + name)
+        ThreadFactoryBuilder().setNameFormat("Log-BackgroundWorker-" + name)
         .build());
     workerExecutor.scheduleWithFixedDelay(new BackgroundWorker(this),
         this.checkpointInterval, this.checkpointInterval,
@@ -355,6 +387,7 @@ public class Log {
   /**
    * Read checkpoint and data files from disk replaying them to the state
    * directly before the shutdown or crash.
+   *
    * @throws IOException
    */
   void replay() throws IOException {
@@ -397,8 +430,8 @@ public class Log {
        * locations. We will read the last one written to disk.
        */
       File checkpointFile = new File(checkpointDir, "checkpoint");
-      if(shouldFastReplay) {
-        if(checkpointFile.exists()) {
+      if (shouldFastReplay) {
+        if (checkpointFile.exists()) {
           LOGGER.debug("Disabling fast full replay because checkpoint " +
               "exists: " + checkpointFile);
           shouldFastReplay = false;
@@ -417,11 +450,12 @@ public class Log {
         backingStore =
             EventQueueBackingStoreFactory.get(checkpointFile,
                 backupCheckpointDir, queueCapacity, channelNameDescriptor,
-                true, this.useDualCheckpoints);
+                channelCounter, true, this.useDualCheckpoints,
+                this.compressBackupCheckpoint);
         queue = new FlumeEventQueue(backingStore, inflightTakesFile,
-                inflightPutsFile, queueSetDir);
+            inflightPutsFile, queueSetDir);
         LOGGER.info("Last Checkpoint " + new Date(checkpointFile.lastModified())
-                + ", queue depth = " + queue.getSize());
+            + ", queue depth = " + queue.getSize());
 
         /*
          * We now have everything we need to actually replay the log files
@@ -440,7 +474,7 @@ public class Log {
               + "Restoring checkpoint and starting up.", ex);
           if (EventQueueBackingStoreFile.backupExists(backupCheckpointDir)) {
             backupRestored = EventQueueBackingStoreFile.restoreBackup(
-              checkpointDir, backupCheckpointDir);
+                checkpointDir, backupCheckpointDir);
           }
         }
         if (!backupRestored) {
@@ -451,16 +485,17 @@ public class Log {
                 "directory to recover from a corrupt or incomplete checkpoint");
           }
         }
-        backingStore = EventQueueBackingStoreFactory.get(checkpointFile,
-            backupCheckpointDir,
-            queueCapacity, channelNameDescriptor, true, useDualCheckpoints);
+        backingStore = EventQueueBackingStoreFactory.get(
+            checkpointFile, backupCheckpointDir, queueCapacity,
+            channelNameDescriptor, channelCounter, true, useDualCheckpoints,
+            compressBackupCheckpoint);
         queue = new FlumeEventQueue(backingStore, inflightTakesFile,
-                inflightPutsFile, queueSetDir);
+            inflightPutsFile, queueSetDir);
         // If the checkpoint was deleted due to BadCheckpointException, then
         // trigger fast replay if the channel is configured to.
         shouldFastReplay = this.useFastReplay;
         doReplay(queue, dataFiles, encryptionKeyProvider, shouldFastReplay);
-        if(!shouldFastReplay) {
+        if (!shouldFastReplay) {
           didFullReplayDueToBadCheckpointException = true;
         }
       }
@@ -493,13 +528,13 @@ public class Log {
                         KeyProvider encryptionKeyProvider,
                         boolean useFastReplay) throws Exception {
     CheckpointRebuilder rebuilder = new CheckpointRebuilder(dataFiles,
-            queue, fsyncPerTransaction);
+        queue, fsyncPerTransaction);
     if (useFastReplay && rebuilder.rebuild()) {
       didFastReplay = true;
       LOGGER.info("Fast replay successful.");
     } else {
       ReplayHandler replayHandler = new ReplayHandler(queue,
-              encryptionKeyProvider, fsyncPerTransaction);
+          encryptionKeyProvider, fsyncPerTransaction);
       if (useLogReplayV1) {
         LOGGER.info("Replaying logs with v1 replay logic");
         replayHandler.replayLogv1(dataFiles);
@@ -519,10 +554,12 @@ public class Log {
   boolean didFastReplay() {
     return didFastReplay;
   }
+
   @VisibleForTesting
   public int getReadCount() {
     return readCount;
   }
+
   @VisibleForTesting
   public int getPutCount() {
     return putCount;
@@ -532,10 +569,12 @@ public class Log {
   public int getTakeCount() {
     return takeCount;
   }
+
   @VisibleForTesting
   public int getCommittedCount() {
     return committedCount;
   }
+
   @VisibleForTesting
   public int getRollbackCount() {
     return rollbackCount;
@@ -543,6 +582,7 @@ public class Log {
 
   /**
    * Was a checkpoint backup used to replay?
+   *
    * @return true if a checkpoint backup was used to replay.
    */
   @VisibleForTesting
@@ -576,7 +616,7 @@ public class Log {
    * @throws InterruptedException
    */
   FlumeEvent get(FlumeEventPointer pointer) throws IOException,
-    InterruptedException, NoopRecordException, CorruptEventException {
+      InterruptedException, NoopRecordException, CorruptEventException {
     Preconditions.checkState(open, "Log is closed");
     int id = pointer.getFileID();
     LogFile.RandomReader logFile = idLogFileMap.get(id);
@@ -587,7 +627,7 @@ public class Log {
       if (fsyncPerTransaction) {
         open = false;
         throw new IOException("Corrupt event found. Please run File Channel " +
-          "Integrity tool.", ex);
+            "Integrity tool.", ex);
       }
       throw ex;
     }
@@ -595,11 +635,12 @@ public class Log {
 
   /**
    * Log a put of an event
-   *
+   * <p>
    * Synchronization not required as this method is atomic
+   *
    * @param transactionID
    * @param event
-   * @return
+   * @return FlumeEventPointer
    * @throws IOException
    */
   FlumeEventPointer put(long transactionID, Event event)
@@ -612,8 +653,8 @@ public class Log {
     int logFileIndex = nextLogWriter(transactionID);
     long usableSpace = logFiles.get(logFileIndex).getUsableSpace();
     long requiredSpace = minimumRequiredSpace + buffer.limit();
-    if(usableSpace <= requiredSpace) {
-      throw new IOException("Usable space exhaused, only " + usableSpace +
+    if (usableSpace <= requiredSpace) {
+      throw new IOException("Usable space exhausted, only " + usableSpace +
           " bytes remaining, required " + requiredSpace + " bytes");
     }
     boolean error = true;
@@ -623,7 +664,7 @@ public class Log {
         error = false;
         return ptr;
       } catch (LogFileRetryableIOException e) {
-        if(!open) {
+        if (!open) {
           throw e;
         }
         roll(logFileIndex, buffer);
@@ -632,7 +673,7 @@ public class Log {
         return ptr;
       }
     } finally {
-      if(error && open) {
+      if (error && open) {
         roll(logFileIndex);
       }
     }
@@ -640,8 +681,9 @@ public class Log {
 
   /**
    * Log a take of an event, pointer points at the corresponding put
-   *
+   * <p>
    * Synchronization not required as this method is atomic
+   *
    * @param transactionID
    * @param pointer
    * @throws IOException
@@ -655,8 +697,8 @@ public class Log {
     int logFileIndex = nextLogWriter(transactionID);
     long usableSpace = logFiles.get(logFileIndex).getUsableSpace();
     long requiredSpace = minimumRequiredSpace + buffer.limit();
-    if(usableSpace <= requiredSpace) {
-      throw new IOException("Usable space exhaused, only " + usableSpace +
+    if (usableSpace <= requiredSpace) {
+      throw new IOException("Usable space exhausted, only " + usableSpace +
           " bytes remaining, required " + requiredSpace + " bytes");
     }
     boolean error = true;
@@ -665,7 +707,7 @@ public class Log {
         logFiles.get(logFileIndex).take(buffer);
         error = false;
       } catch (LogFileRetryableIOException e) {
-        if(!open) {
+        if (!open) {
           throw e;
         }
         roll(logFileIndex, buffer);
@@ -673,7 +715,7 @@ public class Log {
         error = false;
       }
     } finally {
-      if(error && open) {
+      if (error && open) {
         roll(logFileIndex);
       }
     }
@@ -681,15 +723,16 @@ public class Log {
 
   /**
    * Log a rollback of a transaction
-   *
+   * <p>
    * Synchronization not required as this method is atomic
+   *
    * @param transactionID
    * @throws IOException
    */
   void rollback(long transactionID) throws IOException {
     Preconditions.checkState(open, "Log is closed");
 
-    if(LOGGER.isDebugEnabled()) {
+    if (LOGGER.isDebugEnabled()) {
       LOGGER.debug("Rolling back " + transactionID);
     }
     Rollback rollback = new Rollback(transactionID, WriteOrderOracle.next());
@@ -697,8 +740,8 @@ public class Log {
     int logFileIndex = nextLogWriter(transactionID);
     long usableSpace = logFiles.get(logFileIndex).getUsableSpace();
     long requiredSpace = minimumRequiredSpace + buffer.limit();
-    if(usableSpace <= requiredSpace) {
-      throw new IOException("Usable space exhaused, only " + usableSpace +
+    if (usableSpace <= requiredSpace) {
+      throw new IOException("Usable space exhausted, only " + usableSpace +
           " bytes remaining, required " + requiredSpace + " bytes");
     }
     boolean error = true;
@@ -707,7 +750,7 @@ public class Log {
         logFiles.get(logFileIndex).rollback(buffer);
         error = false;
       } catch (LogFileRetryableIOException e) {
-        if(!open) {
+        if (!open) {
           throw e;
         }
         roll(logFileIndex, buffer);
@@ -715,7 +758,7 @@ public class Log {
         error = false;
       }
     } finally {
-      if(error && open) {
+      if (error && open) {
         roll(logFileIndex);
       }
     }
@@ -726,14 +769,15 @@ public class Log {
    * so we know if the pointers corresponding to the events
    * should be added or removed from the flume queue. We
    * could infer but it's best to be explicit.
-   *
+   * <p>
    * Synchronization not required as this method is atomic
+   *
    * @param transactionID
    * @throws IOException
    * @throws InterruptedException
    */
   void commitPut(long transactionID) throws IOException,
-  InterruptedException {
+      InterruptedException {
     Preconditions.checkState(open, "Log is closed");
     commit(transactionID, TransactionEventRecord.Type.PUT.get());
   }
@@ -743,20 +787,21 @@ public class Log {
    * so we know if the pointers corresponding to the events
    * should be added or removed from the flume queue. We
    * could infer but it's best to be explicit.
-   *
+   * <p>
    * Synchronization not required as this method is atomic
+   *
    * @param transactionID
    * @throws IOException
    * @throws InterruptedException
    */
   void commitTake(long transactionID) throws IOException,
-  InterruptedException {
+      InterruptedException {
     Preconditions.checkState(open, "Log is closed");
     commit(transactionID, TransactionEventRecord.Type.TAKE.get());
   }
 
 
-  private void unlockExclusive()  {
+  private void unlockExclusive() {
     checkpointWriterLock.unlock();
   }
 
@@ -764,11 +809,11 @@ public class Log {
     checkpointReadLock.lock();
   }
 
-  void unlockShared()  {
+  void unlockShared() {
     checkpointReadLock.unlock();
   }
 
-  private void lockExclusive(){
+  private void lockExclusive() {
     checkpointWriterLock.lock();
   }
 
@@ -776,15 +821,23 @@ public class Log {
    * Synchronization not required since this method gets the write lock,
    * so checkpoint and this method cannot run at the same time.
    */
-  void close() throws IOException{
+  void close() throws IOException {
     lockExclusive();
     try {
       open = false;
+      try {
+        if (checkpointOnClose) {
+          writeCheckpoint(true); // do this before acquiring exclusive lock
+        }
+      } catch (Exception err) {
+        LOGGER.warn("Failed creating checkpoint on close of channel " + channelNameDescriptor +
+            "Replay will take longer next time channel is started.", err);
+      }
       shutdownWorker();
       if (logFiles != null) {
         for (int index = 0; index < logFiles.length(); index++) {
           LogFile.Writer writer = logFiles.get(index);
-          if(writer != null) {
+          if (writer != null) {
             writer.close();
           }
         }
@@ -833,9 +886,11 @@ public class Log {
       LOGGER.error("Interrupted while waiting for worker to die.");
     }
   }
+
   void setCheckpointInterval(long checkpointInterval) {
     this.checkpointInterval = checkpointInterval;
   }
+
   void setMaxFileSize(long maxFileSize) {
     this.maxFileSize = maxFileSize;
   }
@@ -854,8 +909,8 @@ public class Log {
     int logFileIndex = nextLogWriter(transactionID);
     long usableSpace = logFiles.get(logFileIndex).getUsableSpace();
     long requiredSpace = minimumRequiredSpace + buffer.limit();
-    if(usableSpace <= requiredSpace) {
-      throw new IOException("Usable space exhaused, only " + usableSpace +
+    if (usableSpace <= requiredSpace) {
+      throw new IOException("Usable space exhausted, only " + usableSpace +
           " bytes remaining, required " + requiredSpace + " bytes");
     }
     boolean error = true;
@@ -869,7 +924,7 @@ public class Log {
         logFileWriter.sync();
         error = false;
       } catch (LogFileRetryableIOException e) {
-        if(!open) {
+        if (!open) {
           throw e;
         }
         roll(logFileIndex, buffer);
@@ -879,7 +934,7 @@ public class Log {
         error = false;
       }
     } finally {
-      if(error && open) {
+      if (error && open) {
         roll(logFileIndex);
       }
     }
@@ -888,11 +943,13 @@ public class Log {
 
   /**
    * Atomic so not synchronization required.
+   *
    * @return
    */
   private int nextLogWriter(long transactionID) {
-    return (int)Math.abs(transactionID % (long)logFiles.length());
+    return (int) Math.abs(transactionID % (long) logFiles.length());
   }
+
   /**
    * Unconditionally roll
    * Synchronization done internally
@@ -903,13 +960,14 @@ public class Log {
   private void roll(int index) throws IOException {
     roll(index, null);
   }
+
   /**
    * Roll a log if needed. Roll always occurs if the log at the index
    * does not exist (typically on startup), or buffer is null. Otherwise
    * LogFile.Writer.isRollRequired is checked again to ensure we don't
    * have threads pile up on this log resulting in multiple successive
    * rolls
-   *
+   * <p>
    * Synchronization required since both synchronized and unsynchronized
    * methods call this method, and this method acquires only a
    * read lock. The synchronization guarantees that multiple threads don't
@@ -919,7 +977,7 @@ public class Log {
    * @throws IOException
    */
   private synchronized void roll(int index, ByteBuffer buffer)
-    throws IOException {
+      throws IOException {
     lockShared();
 
     try {
@@ -927,17 +985,17 @@ public class Log {
       // check to make sure a roll is actually required due to
       // the possibility of multiple writes waiting on lock
       if (oldLogFile == null || buffer == null ||
-        oldLogFile.isRollRequired(buffer)) {
+          oldLogFile.isRollRequired(buffer)) {
         try {
           LOGGER.info("Roll start " + logDirs[index]);
           int fileID = nextFileID.incrementAndGet();
           File file = new File(logDirs[index], PREFIX + fileID);
           LogFile.Writer writer = LogFileFactory.getWriter(file, fileID,
-            maxFileSize, encryptionKey, encryptionKeyAlias,
-            encryptionCipherProvider, usableSpaceRefreshInterval,
-            fsyncPerTransaction, fsyncInterval);
+              maxFileSize, encryptionKey, encryptionKeyAlias,
+              encryptionCipherProvider, usableSpaceRefreshInterval,
+              fsyncPerTransaction, fsyncInterval);
           idLogFileMap.put(fileID, LogFileFactory.getRandomReader(file,
-            encryptionKeyProvider, fsyncPerTransaction));
+              encryptionKeyProvider, fsyncPerTransaction));
           // writer from this point on will get new reference
           logFiles.set(index, writer);
           // close out old log
@@ -960,22 +1018,24 @@ public class Log {
   /**
    * Write the current checkpoint object and then swap objects so that
    * the next checkpoint occurs on the other checkpoint directory.
-   *
+   * <p>
    * Synchronization is not required because this method acquires a
    * write lock. So this method gets exclusive access to all the
    * data structures this method accesses.
-   * @param force  a flag to force the writing of checkpoint
+   *
+   * @param force a flag to force the writing of checkpoint
    * @throws IOException if we are unable to write the checkpoint out to disk
    */
   private Boolean writeCheckpoint(Boolean force) throws Exception {
     boolean checkpointCompleted = false;
     long usableSpace = checkpointDir.getUsableSpace();
-    if(usableSpace <= minimumRequiredSpace) {
-      throw new IOException("Usable space exhaused, only " + usableSpace +
+    if (usableSpace <= minimumRequiredSpace) {
+      throw new IOException("Usable space exhausted, only " + usableSpace +
           " bytes remaining, required " + minimumRequiredSpace + " bytes");
     }
     lockExclusive();
-    SortedSet<Integer> logFileRefCountsAll = null, logFileRefCountsActive = null;
+    SortedSet<Integer> logFileRefCountsAll = null;
+    SortedSet<Integer> logFileRefCountsActive = null;
     try {
       if (queue.checkpoint(force)) {
         long logWriteOrderID = queue.getLogWriteOrderID();
@@ -1018,17 +1078,18 @@ public class Log {
           try {
             writer.markCheckpoint(logWriteOrderID);
           } finally {
+            reader = LogFileFactory.getRandomReader(file,
+                encryptionKeyProvider, fsyncPerTransaction);
+            idLogFileMap.put(id, reader);
             writer.close();
           }
-          reader = LogFileFactory.getRandomReader(file,
-            encryptionKeyProvider, fsyncPerTransaction);
-          idLogFileMap.put(id, reader);
+
           LOGGER.debug("Updated checkpoint for file: " + file
               + "logWriteOrderID " + logWriteOrderID);
           idIterator.remove();
         }
         Preconditions.checkState(logFileRefCountsAll.size() == 0,
-                "Could not update all data file timestamps: " + logFileRefCountsAll);
+            "Could not update all data file timestamps: " + logFileRefCountsAll);
         //Add files from all log directories
         for (int index = 0; index < logDirs.length; index++) {
           logFileRefCountsActive.add(logFiles.get(index).getLogFileID());
@@ -1056,7 +1117,7 @@ public class Log {
     // these files) and delete them only after the next (since the current
     // checkpoint will become the backup at that time,
     // and thus these files are no longer needed).
-    for(File fileToDelete : pendingDeletes) {
+    for (File fileToDelete : pendingDeletes) {
       LOGGER.info("Removing old file: " + fileToDelete);
       FileUtils.deleteQuietly(fileToDelete);
     }
@@ -1065,7 +1126,7 @@ public class Log {
     // won't delete any files with an id larger than the min
     int minFileID = fileIDs.first();
     LOGGER.debug("Files currently in use: " + fileIDs);
-    for(File logDir : logDirs) {
+    for (File logDir : logDirs) {
       List<File> logs = LogUtils.getLogs(logDir);
       // sort oldset to newest
       LogUtils.sort(logs);
@@ -1074,9 +1135,9 @@ public class Log {
       for (int index = 0; index < size; index++) {
         File logFile = logs.get(index);
         int logFileID = LogUtils.getIDForFile(logFile);
-        if(logFileID < minFileID) {
+        if (logFileID < minFileID) {
           LogFile.RandomReader reader = idLogFileMap.remove(logFileID);
-          if(reader != null) {
+          if (reader != null) {
             reader.close();
           }
           File metaDataFile = Serialization.getMetaDataFile(logFile);
@@ -1086,13 +1147,14 @@ public class Log {
       }
     }
   }
+
   /**
    * Lock storage to provide exclusive access.
-   *
+   * <p>
    * <p> Locking is not supported by all file systems.
    * E.g., NFS does not consistently support exclusive locks.
-   *
-   * <p> If locking is supported we guarantee exculsive access to the
+   * <p>
+   * <p> If locking is supported we guarantee exclusive access to the
    * storage directory. Otherwise, no guarantee is given.
    *
    * @throws IOException if locking fails
@@ -1107,8 +1169,8 @@ public class Log {
       throw new IOException(msg);
     }
     FileLock secondLock = tryLock(dir);
-    if(secondLock != null) {
-      LOGGER.warn("Directory "+dir+" does not support locking");
+    if (secondLock != null) {
+      LOGGER.warn("Directory " + dir + " does not support locking");
       secondLock.release();
       secondLock.channel().close();
     }
@@ -1130,10 +1192,10 @@ public class Log {
     FileLock res = null;
     try {
       res = file.getChannel().tryLock();
-    } catch(OverlappingFileLockException oe) {
+    } catch (OverlappingFileLockException oe) {
       file.close();
       return null;
-    } catch(IOException e) {
+    } catch (IOException e) {
       LOGGER.error("Cannot create lock on " + lockF, e);
       file.close();
       throw e;
@@ -1148,13 +1210,14 @@ public class Log {
    */
   private void unlock(File dir) throws IOException {
     FileLock lock = locks.remove(dir.getAbsolutePath());
-    if(lock == null) {
+    if (lock == null) {
       return;
     }
     lock.release();
     lock.channel().close();
     lock = null;
   }
+
   static class BackgroundWorker implements Runnable {
     private static final Logger LOG = LoggerFactory
         .getLogger(BackgroundWorker.class);
@@ -1171,8 +1234,10 @@ public class Log {
           log.writeCheckpoint();
         }
       } catch (IOException e) {
+        log.channelCounter.incrementCheckpointWriteErrorCount();
         LOG.error("Error doing checkpoint", e);
       } catch (Throwable e) {
+        log.channelCounter.incrementCheckpointWriteErrorCount();
         LOG.error("General error in checkpoint worker", e);
       }
     }
