@@ -22,16 +22,19 @@ import java.util.concurrent.atomic.AtomicLong;
 
 public final class TransactionIDOracle {
 
-  private TransactionIDOracle() {}
+  private TransactionIDOracle() {
+  }
+
   private static final AtomicLong TRANSACTION_ID =
       new AtomicLong(System.currentTimeMillis());
 
   public static void setSeed(long highest) {
     long previous;
-    while(highest > (previous = TRANSACTION_ID.get())) {
+    while (highest > (previous = TRANSACTION_ID.get())) {
       TRANSACTION_ID.compareAndSet(previous, highest);
     }
   }
+
   public static long next() {
     return TRANSACTION_ID.incrementAndGet();
   }
