@@ -19,45 +19,65 @@
 
 package org.apache.flume.client.avro;
 
-import java.io.File;
-import java.io.FileFilter;
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.nio.charset.Charset;
-import java.util.*;
-import java.util.regex.Pattern;
-
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Charsets;
+import com.google.common.base.Optional;
+import com.google.common.base.Preconditions;
 import org.apache.flume.Context;
 import org.apache.flume.Event;
 import org.apache.flume.FlumeException;
 import org.apache.flume.annotations.InterfaceAudience;
 import org.apache.flume.annotations.InterfaceStability;
-import org.apache.flume.serialization.*;
+import org.apache.flume.event.EventBuilder;
+import org.apache.flume.instrumentation.SourceCounter;
+import org.apache.flume.serialization.DecodeErrorPolicy;
+import org.apache.flume.serialization.DurablePositionTracker;
+import org.apache.flume.serialization.EventDeserializer;
+import org.apache.flume.serialization.EventDeserializerFactory;
+import org.apache.flume.serialization.PositionTracker;
+import org.apache.flume.serialization.ResettableFileInputStream;
+import org.apache.flume.serialization.ResettableInputStream;
 import org.apache.flume.source.SpoolDirectorySourceConfigurationConstants;
+import org.apache.flume.source.SpoolDirectorySourceConfigurationConstants.ConsumeOrder;
 import org.apache.flume.tools.PlatformDetect;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.base.Charsets;
-import com.google.common.base.Optional;
-import com.google.common.base.Preconditions;
-import com.google.common.io.Files;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.nio.charset.Charset;
+import java.nio.file.FileVisitResult;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
+import java.nio.file.SimpleFileVisitor;
+import java.nio.file.attribute.BasicFileAttributes;
+import java.time.Instant;
+import java.time.ZoneId;
+import java.time.format.DateTimeFormatter;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Locale;
+import java.util.Set;
+import java.util.regex.Pattern;
 
 /**
- * <p/>A {@link ReliableEventReader} which reads log data from files stored
+ * <p>A {@link ReliableEventReader} which reads log data from files stored
  * in a spooling directory and renames each file once all of its data has been
  * read (through {@link EventDeserializer#readEvent()} calls). The user must
  * {@link #commit()} each read, to indicate that the lines have been fully
  * processed.
- * <p/>Read calls will return no data if there are no files left to read. This
+ * <p>Read calls will return no data if there are no files left to read. This
  * class, in general, is not thread safe.
- *
- * <p/>This reader assumes that files with unique file names are left in the
+ * <p>This reader assumes that files with unique file names are left in the
  * spooling directory and not modified once they are placed there. Any user
  * behavior which violates these assumptions, when detected, will result in a
  * FlumeException being thrown.
- *
- * <p/>This class makes the following guarantees, if above assumptions are met:
+ * <p>This class makes the following guarantees, if above assumptions are met:
  * <ul>
  * <li> Once a log file has been renamed with the {@link #completedSuffix},
  *      all of its records have been read through the
@@ -75,41 +95,65 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
       .getLogger(ReliableSpoolingFileEventReader.class);
 
   static final String metaFileName = ".flumespool-main.meta";
-
+  private static final DateTimeFormatter FORMATTER = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss.SSS");
   private final File spoolDirectory;
+  private final Path spoolDirPath;
   private final String completedSuffix;
   private final String deserializerType;
   private final Context deserializerContext;
+  private final Pattern includePattern;
   private final Pattern ignorePattern;
   private final File metaFile;
+  private File trackerDirectory;
   private final boolean annotateFileName;
+  private final boolean annotateBaseName;
   private final String fileNameHeader;
+  private final String baseNameHeader;
   private final String deletePolicy;
+  private final TrackingPolicy trackingPolicy;
   private final Charset inputCharset;
+  private final DecodeErrorPolicy decodeErrorPolicy;
+  private final ConsumeOrder consumeOrder;
+  private final boolean recursiveDirectorySearch;
+  private final SourceCounter sourceCounter;
 
   private Optional<FileInfo> currentFile = Optional.absent();
-  /** Always contains the last file from which lines have been read. **/
+  /** Always contains the last file from which lines have been read. */
   private Optional<FileInfo> lastFileRead = Optional.absent();
   private boolean committed = true;
+  private boolean firstTimeRead = true;
+
+  /** Instance var to Cache directory listing */
+  private Iterator<File> candidateFileIter = null;
+  private int listFilesCount = 0;
+
+  private String trackerDirectoryAbsolutePath;
 
   /**
    * Create a ReliableSpoolingFileEventReader to watch the given directory.
    */
   private ReliableSpoolingFileEventReader(File spoolDirectory,
-      String completedSuffix, String ignorePattern, String trackerDirPath,
+      String completedSuffix, String includePattern, String ignorePattern, String trackerDirPath,
       boolean annotateFileName, String fileNameHeader,
+      boolean annotateBaseName, String baseNameHeader,
       String deserializerType, Context deserializerContext,
-      String deletePolicy, String inputCharset) throws IOException {
+      String deletePolicy, String trackingPolicy, String inputCharset,
+      DecodeErrorPolicy decodeErrorPolicy,
+      ConsumeOrder consumeOrder, boolean recursiveDirectorySearch,
+                                          SourceCounter sourceCounter) throws IOException {
 
     // Sanity checks
     Preconditions.checkNotNull(spoolDirectory);
     Preconditions.checkNotNull(completedSuffix);
+    Preconditions.checkNotNull(includePattern);
     Preconditions.checkNotNull(ignorePattern);
     Preconditions.checkNotNull(trackerDirPath);
     Preconditions.checkNotNull(deserializerType);
     Preconditions.checkNotNull(deserializerContext);
     Preconditions.checkNotNull(deletePolicy);
+    Preconditions.checkNotNull(trackingPolicy);
     Preconditions.checkNotNull(inputCharset);
+    Preconditions.checkNotNull(sourceCounter);
 
     // validate delete policy
     if (!deletePolicy.equalsIgnoreCase(DeletePolicy.NEVER.name()) &&
@@ -118,11 +162,20 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
           "NEVER and IMMEDIATE are not yet supported");
     }
 
+    // validate tracking policy
+    if (!trackingPolicy.equalsIgnoreCase(TrackingPolicy.RENAME.name()) &&
+            !trackingPolicy.equalsIgnoreCase(TrackingPolicy.TRACKER_DIR.name())) {
+      throw new IllegalArgumentException("Tracking policies other than " +
+              "RENAME and TRACKER_DIR are not supported");
+    }
+
     if (logger.isDebugEnabled()) {
       logger.debug("Initializing {} with directory={}, metaDir={}, " +
-          "deserializer={}",
-          new Object[] { ReliableSpoolingFileEventReader.class.getSimpleName(),
-          spoolDirectory, trackerDirPath, deserializerType });
+                   "deserializer={}",
+                   new Object[] {
+                     ReliableSpoolingFileEventReader.class.getSimpleName(),
+                     spoolDirectory, trackerDirPath, deserializerType
+                   });
     }
 
     // Verify directory exists and is readable/writable
@@ -131,17 +184,22 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     Preconditions.checkState(spoolDirectory.isDirectory(),
         "Path is not a directory: " + spoolDirectory.getAbsolutePath());
 
-    // Do a canary test to make sure we have access to spooling directory
-    try {
-      File f1 = File.createTempFile("flume", "test", spoolDirectory);
-      Files.write("testing flume file permissions\n", f1, Charsets.UTF_8);
-      Files.readLines(f1, Charsets.UTF_8);
-      if (!f1.delete()) {
-        throw new IOException("Unable to delete canary file " + f1);
+    if(trackingPolicy.equalsIgnoreCase(TrackingPolicy.RENAME.name())) {
+      // Do a canary test to make sure we have access to spooling directory
+      try {
+        File canary = File.createTempFile("flume-spooldir-perm-check-", ".canary",
+                spoolDirectory);
+        Files.write(canary.toPath(), "testing flume file permissions\n".getBytes());
+        List<String> lines = Files.readAllLines(canary.toPath(), Charsets.UTF_8);
+        Preconditions.checkState(!lines.isEmpty(), "Empty canary file %s", canary);
+        if (!canary.delete()) {
+          throw new IOException("Unable to delete canary file " + canary);
+        }
+        logger.debug("Successfully created and deleted canary file: {}", canary);
+      } catch (IOException e) {
+        throw new FlumeException("Unable to read and modify files" +
+                " in the spooling directory: " + spoolDirectory, e);
       }
-    } catch (IOException e) {
-      throw new FlumeException("Unable to read and modify files" +
-          " in the spooling directory: " + spoolDirectory, e);
     }
 
     this.spoolDirectory = spoolDirectory;
@@ -150,11 +208,19 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     this.deserializerContext = deserializerContext;
     this.annotateFileName = annotateFileName;
     this.fileNameHeader = fileNameHeader;
+    this.annotateBaseName = annotateBaseName;
+    this.baseNameHeader = baseNameHeader;
+    this.includePattern = Pattern.compile(includePattern);
     this.ignorePattern = Pattern.compile(ignorePattern);
     this.deletePolicy = deletePolicy;
+    this.trackingPolicy = TrackingPolicy.valueOf(trackingPolicy.toUpperCase());
     this.inputCharset = Charset.forName(inputCharset);
+    this.decodeErrorPolicy = Preconditions.checkNotNull(decodeErrorPolicy);
+    this.consumeOrder = Preconditions.checkNotNull(consumeOrder);
+    this.recursiveDirectorySearch = recursiveDirectorySearch;
+    this.sourceCounter = sourceCounter;
 
-    File trackerDirectory = new File(trackerDirPath);
+    trackerDirectory = new File(trackerDirPath);
 
     // if relative path, treat as relative to spool directory
     if (!trackerDirectory.isAbsolute()) {
@@ -176,11 +242,109 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     }
 
     this.metaFile = new File(trackerDirectory, metaFileName);
+
+    if (metaFile.exists() && metaFile.length() == 0) {
+      deleteMetaFile();
+    }
+
+    spoolDirPath = Paths.get(spoolDirectory.getAbsolutePath());
+    trackerDirectoryAbsolutePath = trackerDirectory.getAbsolutePath();
+
   }
 
-  /** Return the filename which generated the data from the last successful
+  /**
+   * Recursively gather candidate files
+   *
+   * @param directory the directory to gather files from
+   * @return list of files within the passed in directory
+   */
+  private List<File> getCandidateFiles(final Path directory) {
+    Preconditions.checkNotNull(directory);
+    final List<File> candidateFiles = new ArrayList<>();
+    try {
+      final Set<Path> trackerDirCompletedFiles = getTrackerDirCompletedFiles();
+      Files.walkFileTree(directory, new SimpleFileVisitor<Path>() {
+        @Override
+        public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs)
+            throws IOException {
+          if (directory.equals(dir)) { // The top directory should always be listed
+            return FileVisitResult.CONTINUE;
+          }
+          String directoryName = dir.getFileName().toString();
+          if (!recursiveDirectorySearch ||
+              directoryName.startsWith(".") ||
+              ignorePattern.matcher(directoryName).matches()) {
+            return FileVisitResult.SKIP_SUBTREE;
+          }
+          return FileVisitResult.CONTINUE;
+        }
+
+        @Override
+        public FileVisitResult visitFile(Path candidate, BasicFileAttributes attrs)
+            throws IOException {
+          String fileName = candidate.getFileName().toString();
+          if (!fileName.endsWith(completedSuffix) &&
+              !isFileInTrackerDir(trackerDirCompletedFiles, candidate) &&
+              !fileName.startsWith(".") &&
+              includePattern.matcher(fileName).matches() &&
+              !ignorePattern.matcher(fileName).matches()) {
+            candidateFiles.add(candidate.toFile());
+          }
+
+          return FileVisitResult.CONTINUE;
+        }
+      });
+    } catch (IOException e) {
+      logger.error("I/O exception occurred while listing directories. " +
+                   "Files already matched will be returned. " + directory, e);
+      sourceCounter.incrementGenericProcessingFail();
+    }
+
+    return candidateFiles;
+  }
+
+  private Set<Path> getTrackerDirCompletedFiles() throws IOException {
+    final Set<Path> completedFiles = new HashSet<>();
+    if (TrackingPolicy.TRACKER_DIR != trackingPolicy) {
+      return completedFiles;
+    }
+
+    Path trackerDirPath = trackerDirectory.toPath();
+    Files.walkFileTree(trackerDirPath, new SimpleFileVisitor<Path>() {
+
+      @Override
+      public FileVisitResult visitFile(Path candidate, BasicFileAttributes attrs)
+              throws IOException {
+        String fileName = candidate.getFileName().toString();
+        if (fileName.endsWith(completedSuffix)) {
+          completedFiles.add(candidate.toAbsolutePath());
+        }
+        return FileVisitResult.CONTINUE;
+      }
+    });
+    return completedFiles;
+  }
+
+  private boolean isFileInTrackerDir(Set<Path> completedFiles, Path path) {
+    Path relPath = getRelPathToSpoolDir(path);
+    Path trackerPath = Paths.get(trackerDirectoryAbsolutePath, relPath.toString() + completedSuffix);
+    return completedFiles.contains(trackerPath);
+  }
+
+  private Path getRelPathToSpoolDir(Path path) {
+    return spoolDirPath.relativize(path.toAbsolutePath());
+  }
+
+  @VisibleForTesting
+  int getListFilesCount() {
+    return listFilesCount;
+  }
+
+  /**
+   * Return the filename which generated the data from the last successful
    * {@link #readEvents(int)} call. Returns null if called before any file
-   * contents are read. */
+   * contents are read.
+   */
   public String getLastFileRead() {
     if (!lastFileRead.isPresent()) {
       return null;
@@ -188,6 +352,23 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     return lastFileRead.get().getFile().getAbsolutePath();
   }
 
+  /**
+   * Return the filename, lastModified, and size which generated the data from the last successful
+   * {@link #readEvents(int)} call. Returns null if called before any file
+   * contents are read.
+   */
+  public String getLastFileReadInfo() {
+    if (!lastFileRead.isPresent()) {
+      return null;
+    }
+    FileInfo fileInfo = lastFileRead.get();
+    StringBuilder sb = new StringBuilder(fileInfo.getFile().getName());
+    sb.append(" lastModified: ").append(FORMATTER.format(Instant.ofEpochMilli(fileInfo.lastModified)
+        .atZone(ZoneId.systemDefault()).toLocalDateTime()));
+    sb.append(" size: ").append(fileInfo.length);
+    return sb.toString();
+  }
+
   // public interface
   public Event readEvent() throws IOException {
     List<Event> events = readEvents(1);
@@ -217,20 +398,40 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
       }
     }
 
-    EventDeserializer des = currentFile.get().getDeserializer();
-    List<Event> events = des.readEvents(numEvents);
+    List<Event> events = readDeserializerEvents(numEvents);
 
     /* It's possible that the last read took us just up to a file boundary.
-     * If so, try to roll to the next file, if there is one. */
-    if (events.isEmpty()) {
+     * If so, try to roll to the next file, if there is one.
+     * Loop until events is not empty or there is no next file in case of 0 byte files */
+    while (events.isEmpty()) {
+      logger.info("Last read took us just up to a file boundary. " +
+                  "Rolling to the next file, if there is one.");
       retireCurrentFile();
       currentFile = getNextFile();
       if (!currentFile.isPresent()) {
         return Collections.emptyList();
       }
-      events = currentFile.get().getDeserializer().readEvents(numEvents);
+      events = readDeserializerEvents(numEvents);
     }
 
+    fillHeader(events);
+
+    committed = false;
+    lastFileRead = currentFile;
+    return events;
+  }
+
+  private List<Event> readDeserializerEvents(int numEvents) throws IOException {
+    EventDeserializer des = currentFile.get().getDeserializer();
+    List<Event> events = des.readEvents(numEvents);
+    if (events.isEmpty() && firstTimeRead) {
+      events.add(EventBuilder.withBody(new byte[0]));
+    }
+    firstTimeRead = false;
+    return events;
+  }
+
+  private void fillHeader(List<Event> events) {
     if (annotateFileName) {
       String filename = currentFile.get().getFile().getAbsolutePath();
       for (Event event : events) {
@@ -238,9 +439,12 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
       }
     }
 
-    committed = false;
-    lastFileRead = currentFile;
-    return events;
+    if (annotateBaseName) {
+      String basename = currentFile.get().getFile().getName();
+      for (Event event : events) {
+        event.getHeaders().put(baseNameHeader, basename);
+      }
+    }
   }
 
   @Override
@@ -251,7 +455,9 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     }
   }
 
-  /** Commit the last lines which were read. */
+  /**
+   * Commit the last lines which were read.
+   */
   @Override
   public void commit() throws IOException {
     if (!committed && currentFile.isPresent()) {
@@ -262,11 +468,12 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
 
   /**
    * Closes currentFile and attempt to rename it.
-   *
+   * <p>
    * If these operations fail in a way that may cause duplicate log entries,
    * an error is logged but no exceptions are thrown. If these operations fail
    * in a way that indicates potential misuse of the spooling directory, a
    * FlumeException will be thrown.
+   *
    * @throws FlumeException if files do not conform to spooling assumptions
    */
   private void retireCurrentFile() throws IOException {
@@ -287,7 +494,11 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     }
 
     if (deletePolicy.equalsIgnoreCase(DeletePolicy.NEVER.name())) {
-      rollCurrentFile(fileToRoll);
+      if (trackingPolicy == TrackingPolicy.RENAME) {
+        rollCurrentFile(fileToRoll);
+      } else {
+        rollCurrentFileInTrackerDir(fileToRoll);
+      }
     } else if (deletePolicy.equalsIgnoreCase(DeletePolicy.IMMEDIATE.name())) {
       deleteCurrentFile(fileToRoll);
     } else {
@@ -299,6 +510,7 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
 
   /**
    * Rename the given spooled file
+   *
    * @param fileToRoll
    * @throws IOException
    */
@@ -317,13 +529,14 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
        * file was already rolled but the rename was not atomic. If that seems
        * likely, we let it pass with only a warning.
        */
-      if (Files.equal(currentFile.get().getFile(), dest)) {
+      if (com.google.common.io.Files.equal(currentFile.get().getFile(), dest)) {
         logger.warn("Completed file " + dest +
             " already exists, but files match, so continuing.");
         boolean deleted = fileToRoll.delete();
         if (!deleted) {
           logger.error("Unable to delete file " + fileToRoll.getAbsolutePath() +
               ". It will likely be ingested another time.");
+          sourceCounter.incrementGenericProcessingFail();
         }
       } else {
         String message = "File name has been re-used with different" +
@@ -331,13 +544,13 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
         throw new IllegalStateException(message);
       }
 
-    // Dest file exists and not on windows
+      // Dest file exists and not on windows
     } else if (dest.exists()) {
       String message = "File name has been re-used with different" +
           " files. Spooling assumptions violated for " + dest;
       throw new IllegalStateException(message);
 
-    // Destination file does not already exist. We are good to go!
+      // Destination file does not already exist. We are good to go!
     } else {
       boolean renamed = fileToRoll.renameTo(dest);
       if (renamed) {
@@ -357,8 +570,27 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     }
   }
 
+  private void rollCurrentFileInTrackerDir(File fileToRoll) throws IOException {
+    Path path = fileToRoll.toPath();
+    Path relToRoll = getRelPathToSpoolDir(path);
+
+    File dest = new File(trackerDirectory.getPath(), relToRoll + completedSuffix);
+    logger.info("Preparing to create tracker file for {} at {}", fileToRoll, dest);
+    if (dest.exists()) {
+      String message = "File name has been re-used with different" +
+              " files. Spooling assumptions violated for " + dest;
+      throw new IllegalStateException(message);
+    }
+    //Create an empty file as an indicator
+    dest.getParentFile().mkdirs(); //create the parent dirs first
+    if (!dest.createNewFile()) {
+      throw new IOException("Could not create tracker file: " + dest);
+    }
+  }
+
   /**
    * Delete the given spooled file
+   *
    * @param fileToDelete
    * @throws IOException
    */
@@ -376,72 +608,110 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
   }
 
   /**
-   * Find and open the oldest file in the chosen directory. If two or more
-   * files are equally old, the file name with lower lexicographical value is
-   * returned. If the directory is empty, this will return an absent option.
+   * Returns the next file to be consumed from the chosen directory.
+   * If the directory is empty or the chosen file is not readable,
+   * this will return an absent option.
+   * If the {@link #consumeOrder} variable is {@link ConsumeOrder#OLDEST}
+   * then returns the oldest file. If the {@link #consumeOrder} variable
+   * is {@link ConsumeOrder#YOUNGEST} then returns the youngest file.
+   * If two or more files are equally old/young, then the file name with
+   * lower lexicographical value is returned.
+   * If the {@link #consumeOrder} variable is {@link ConsumeOrder#RANDOM}
+   * then cache the directory listing to amortize retreival cost, and return
+   * any arbitary file from the directory.
    */
   private Optional<FileInfo> getNextFile() {
-    /* Filter to exclude finished or hidden files */
-    FileFilter filter = new FileFilter() {
-      public boolean accept(File candidate) {
-        String fileName = candidate.getName();
-        if ((candidate.isDirectory()) ||
-            (fileName.endsWith(completedSuffix)) ||
-            (fileName.startsWith(".")) ||
-            ignorePattern.matcher(fileName).matches()) {
-          return false;
-        }
-        return true;
-      }
-    };
-    List<File> candidateFiles = Arrays.asList(spoolDirectory.listFiles(filter));
-    if (candidateFiles.isEmpty()) {
+    List<File> candidateFiles = Collections.emptyList();
+
+    if (consumeOrder != ConsumeOrder.RANDOM ||
+        candidateFileIter == null ||
+        !candidateFileIter.hasNext()) {
+      candidateFiles = getCandidateFiles(spoolDirectory.toPath());
+      listFilesCount++;
+      candidateFileIter = candidateFiles.iterator();
+    }
+
+    if (!candidateFileIter.hasNext()) { // No matching file in spooling directory.
       return Optional.absent();
-    } else {
-      Collections.sort(candidateFiles, new Comparator<File>() {
-        public int compare(File a, File b) {
-          int timeComparison = new Long(a.lastModified()).compareTo(
-              new Long(b.lastModified()));
-          if (timeComparison != 0) {
-            return timeComparison;
-          }
-          else {
-            return a.getName().compareTo(b.getName());
-          }
+    }
+
+    File selectedFile = candidateFileIter.next();
+    if (consumeOrder == ConsumeOrder.RANDOM) { // Selected file is random.
+      return openFile(selectedFile);
+    } else if (consumeOrder == ConsumeOrder.YOUNGEST) {
+      for (File candidateFile : candidateFiles) {
+        long compare = selectedFile.lastModified() -
+            candidateFile.lastModified();
+        if (compare == 0) { // ts is same pick smallest lexicographically.
+          selectedFile = smallerLexicographical(selectedFile, candidateFile);
+        } else if (compare < 0) { // candidate is younger (cand-ts > selec-ts)
+          selectedFile = candidateFile;
         }
-      });
-      File nextFile = candidateFiles.get(0);
-      try {
-        // roll the meta file, if needed
-        String nextPath = nextFile.getPath();
-        PositionTracker tracker =
-            DurablePositionTracker.getInstance(metaFile, nextPath);
-        if (!tracker.getTarget().equals(nextPath)) {
-          tracker.close();
-          deleteMetaFile();
-          tracker = DurablePositionTracker.getInstance(metaFile, nextPath);
-        }
-
-        // sanity check
-        Preconditions.checkState(tracker.getTarget().equals(nextPath),
-            "Tracker target %s does not equal expected filename %s",
-            tracker.getTarget(), nextPath);
-
-        ResettableInputStream in =
-            new ResettableFileInputStream(nextFile, tracker,
-                ResettableFileInputStream.DEFAULT_BUF_SIZE, inputCharset);
-        EventDeserializer deserializer = EventDeserializerFactory.getInstance
-            (deserializerType, deserializerContext, in);
-
-        return Optional.of(new FileInfo(nextFile, deserializer));
-      } catch (FileNotFoundException e) {
-        // File could have been deleted in the interim
-        logger.warn("Could not find file: " + nextFile, e);
-        return Optional.absent();
-      } catch (IOException e) {
-        logger.error("Exception opening file: " + nextFile, e);
-        return Optional.absent();
       }
+    } else { // default order is OLDEST
+      for (File candidateFile : candidateFiles) {
+        long compare = selectedFile.lastModified() -
+            candidateFile.lastModified();
+        if (compare == 0) { // ts is same pick smallest lexicographically.
+          selectedFile = smallerLexicographical(selectedFile, candidateFile);
+        } else if (compare > 0) { // candidate is older (cand-ts < selec-ts).
+          selectedFile = candidateFile;
+        }
+      }
+    }
+
+    firstTimeRead = true;
+
+    return openFile(selectedFile);
+  }
+
+  private File smallerLexicographical(File f1, File f2) {
+    if (f1.getName().compareTo(f2.getName()) < 0) {
+      return f1;
+    }
+    return f2;
+  }
+
+  /**
+   * Opens a file for consuming
+   *
+   * @param file
+   * @return {@link FileInfo} for the file to consume or absent option if the
+   * file does not exists or readable.
+   */
+  private Optional<FileInfo> openFile(File file) {
+    try {
+      // roll the meta file, if needed
+      String nextPath = file.getPath();
+      PositionTracker tracker =
+          DurablePositionTracker.getInstance(metaFile, nextPath);
+      if (!tracker.getTarget().equals(nextPath)) {
+        tracker.close();
+        deleteMetaFile();
+        tracker = DurablePositionTracker.getInstance(metaFile, nextPath);
+      }
+
+      // sanity check
+      Preconditions.checkState(tracker.getTarget().equals(nextPath),
+          "Tracker target %s does not equal expected filename %s",
+          tracker.getTarget(), nextPath);
+
+      ResettableInputStream in =
+          new ResettableFileInputStream(file, tracker,
+              ResettableFileInputStream.DEFAULT_BUF_SIZE, inputCharset,
+              decodeErrorPolicy);
+      EventDeserializer deserializer =
+          EventDeserializerFactory.getInstance(deserializerType, deserializerContext, in);
+
+      return Optional.of(new FileInfo(file, deserializer));
+    } catch (FileNotFoundException e) {
+      // File could have been deleted in the interim
+      logger.warn("Could not find file: " + file, e);
+      return Optional.absent();
+    } catch (IOException e) {
+      logger.error("Exception opening file: " + file, e);
+      sourceCounter.incrementGenericProcessingFail();
+      return Optional.absent();
     }
   }
 
@@ -451,7 +721,9 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     }
   }
 
-  /** An immutable class with information about a file being processed. */
+  /**
+   * An immutable class with information about a file being processed.
+   */
   private static class FileInfo {
     private final File file;
     private final long length;
@@ -465,20 +737,39 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
       this.deserializer = deserializer;
     }
 
-    public long getLength() { return length; }
-    public long getLastModified() { return lastModified; }
-    public EventDeserializer getDeserializer() { return deserializer; }
-    public File getFile() { return file; }
+    public long getLength() {
+      return length;
+    }
+
+    public long getLastModified() {
+      return lastModified;
+    }
+
+    public EventDeserializer getDeserializer() {
+      return deserializer;
+    }
+
+    public File getFile() {
+      return file;
+    }
   }
 
   @InterfaceAudience.Private
   @InterfaceStability.Unstable
-  static enum DeletePolicy {
+  enum DeletePolicy {
     NEVER,
     IMMEDIATE,
     DELAY
   }
 
+  @InterfaceAudience.Private
+  @InterfaceStability.Unstable
+  public enum TrackingPolicy {
+    RENAME,
+    TRACKER_DIR
+  }
+
+
   /**
    * Special builder class for ReliableSpoolingFileEventReader
    */
@@ -486,6 +777,8 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
     private File spoolDirectory;
     private String completedSuffix =
         SpoolDirectorySourceConfigurationConstants.SPOOLED_FILE_SUFFIX;
+    private String includePattern = 
+        SpoolDirectorySourceConfigurationConstants.DEFAULT_INCLUDE_PAT;
     private String ignorePattern =
         SpoolDirectorySourceConfigurationConstants.DEFAULT_IGNORE_PAT;
     private String trackerDirPath =
@@ -494,13 +787,27 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
         SpoolDirectorySourceConfigurationConstants.DEFAULT_FILE_HEADER;
     private String fileNameHeader =
         SpoolDirectorySourceConfigurationConstants.DEFAULT_FILENAME_HEADER_KEY;
+    private Boolean annotateBaseName =
+        SpoolDirectorySourceConfigurationConstants.DEFAULT_BASENAME_HEADER;
+    private String baseNameHeader =
+        SpoolDirectorySourceConfigurationConstants.DEFAULT_BASENAME_HEADER_KEY;
     private String deserializerType =
         SpoolDirectorySourceConfigurationConstants.DEFAULT_DESERIALIZER;
     private Context deserializerContext = new Context();
     private String deletePolicy =
         SpoolDirectorySourceConfigurationConstants.DEFAULT_DELETE_POLICY;
+    private String trackingPolicy =
+            SpoolDirectorySourceConfigurationConstants.DEFAULT_TRACKING_POLICY;
     private String inputCharset =
         SpoolDirectorySourceConfigurationConstants.DEFAULT_INPUT_CHARSET;
+    private DecodeErrorPolicy decodeErrorPolicy = DecodeErrorPolicy.valueOf(
+        SpoolDirectorySourceConfigurationConstants.DEFAULT_DECODE_ERROR_POLICY
+            .toUpperCase(Locale.ENGLISH));
+    private ConsumeOrder consumeOrder =
+        SpoolDirectorySourceConfigurationConstants.DEFAULT_CONSUME_ORDER;
+    private boolean recursiveDirectorySearch =
+        SpoolDirectorySourceConfigurationConstants.DEFAULT_RECURSIVE_DIRECTORY_SEARCH;
+    private SourceCounter sourceCounter;
 
     public Builder spoolDirectory(File directory) {
       this.spoolDirectory = directory;
@@ -512,6 +819,11 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
       return this;
     }
 
+    public Builder includePattern(String includePattern) {
+      this.includePattern = includePattern;
+      return this;
+    }
+
     public Builder ignorePattern(String ignorePattern) {
       this.ignorePattern = ignorePattern;
       return this;
@@ -532,6 +844,16 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
       return this;
     }
 
+    public Builder annotateBaseName(Boolean annotateBaseName) {
+      this.annotateBaseName = annotateBaseName;
+      return this;
+    }
+
+    public Builder baseNameHeader(String baseNameHeader) {
+      this.baseNameHeader = baseNameHeader;
+      return this;
+    }
+
     public Builder deserializerType(String deserializerType) {
       this.deserializerType = deserializerType;
       return this;
@@ -547,15 +869,42 @@ public class ReliableSpoolingFileEventReader implements ReliableEventReader {
       return this;
     }
 
+    public Builder trackingPolicy(String trackingPolicy) {
+      this.trackingPolicy = trackingPolicy;
+      return this;
+    }
+
     public Builder inputCharset(String inputCharset) {
       this.inputCharset = inputCharset;
       return this;
     }
 
+    public Builder recursiveDirectorySearch(boolean recursiveDirectorySearch) {
+      this.recursiveDirectorySearch = recursiveDirectorySearch;
+      return this;
+    }
+
+    public Builder decodeErrorPolicy(DecodeErrorPolicy decodeErrorPolicy) {
+      this.decodeErrorPolicy = decodeErrorPolicy;
+      return this;
+    }
+
+    public Builder consumeOrder(ConsumeOrder consumeOrder) {
+      this.consumeOrder = consumeOrder;
+      return this;
+    }
+
+    public Builder sourceCounter(SourceCounter sourceCounter) {
+      this.sourceCounter = sourceCounter;
+      return this;
+    }
+
     public ReliableSpoolingFileEventReader build() throws IOException {
       return new ReliableSpoolingFileEventReader(spoolDirectory, completedSuffix,
-          ignorePattern, trackerDirPath, annotateFileName, fileNameHeader,
-          deserializerType, deserializerContext, deletePolicy, inputCharset);
+          includePattern, ignorePattern, trackerDirPath, annotateFileName, fileNameHeader,
+          annotateBaseName, baseNameHeader, deserializerType,
+          deserializerContext, deletePolicy, trackingPolicy, inputCharset, decodeErrorPolicy,
+          consumeOrder, recursiveDirectorySearch, sourceCounter);
     }
   }
 
