@@ -29,9 +29,9 @@ import org.apache.flume.annotations.InterfaceStability;
  */
 @InterfaceAudience.Public
 @InterfaceStability.Evolving
-public abstract class AbstractEventDrivenSource extends BasicSourceSemantics
-  implements EventDrivenSource {
-
+public abstract class AbstractEventDrivenSource
+    extends BasicSourceSemantics
+    implements EventDrivenSource {
   public AbstractEventDrivenSource() {
     super();
   }
