@@ -20,7 +20,6 @@ package org.apache.flume.source;
 
 import com.google.common.base.Preconditions;
 import com.google.common.base.Throwables;
-import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
 
 import java.io.IOException;
@@ -122,7 +121,7 @@ public class MultiportSyslogTCPSource extends SslContextAwareAbstractSource impl
     // clear any previous charset configuration and reconfigure it
     portCharsets.clear();
     {
-      ImmutableMap<String, String> portCharsetCfg = context.getSubProperties(
+      Map<String, String> portCharsetCfg = context.getSubProperties(
           SyslogSourceConfigurationConstants.CONFIG_PORT_CHARSET_PREFIX);
       for (Map.Entry<String, String> entry : portCharsetCfg.entrySet()) {
         String portStr = entry.getKey();
