@@ -20,8 +20,14 @@ package org.apache.flume.source;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Throwables;
-import org.apache.flume.*;
+import org.apache.flume.ChannelException;
+import org.apache.flume.ChannelFullException;
+import org.apache.flume.Context;
+import org.apache.flume.Event;
+import org.apache.flume.EventDrivenSource;
+import org.apache.flume.FlumeException;
 import org.apache.flume.client.avro.ReliableSpoolingFileEventReader;
+import org.apache.flume.conf.BatchSizeSupported;
 import org.apache.flume.conf.Configurable;
 import org.apache.flume.instrumentation.SourceCounter;
 import org.apache.flume.serialization.DecodeErrorPolicy;
@@ -32,20 +38,17 @@ import org.slf4j.LoggerFactory;
 import java.io.File;
 import java.io.IOException;
 import java.util.List;
+import java.util.Locale;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 
 import static org.apache.flume.source.SpoolDirectorySourceConfigurationConstants.*;
 
-public class SpoolDirectorySource extends AbstractSource implements
-Configurable, EventDrivenSource {
+public class SpoolDirectorySource extends AbstractSource
+    implements Configurable, EventDrivenSource, BatchSizeSupported {
 
-  private static final Logger logger = LoggerFactory
-      .getLogger(SpoolDirectorySource.class);
-
-  // Delay used when polling for new files
-  private static final int POLL_DELAY_MS = 500;
+  private static final Logger logger = LoggerFactory.getLogger(SpoolDirectorySource.class);
 
   /* Config options */
   private String completedSuffix;
@@ -55,6 +58,7 @@ Configurable, EventDrivenSource {
   private boolean basenameHeader;
   private String basenameHeaderKey;
   private int batchSize;
+  private String includePattern;
   private String ignorePattern;
   private String trackerDirPath;
   private String deserializerType;
@@ -69,8 +73,12 @@ Configurable, EventDrivenSource {
   private ScheduledExecutorService executor;
   private boolean backoff = true;
   private boolean hitChannelException = false;
+  private boolean hitChannelFullException = false;
   private int maxBackoff;
   private ConsumeOrder consumeOrder;
+  private int pollDelay;
+  private boolean recursiveDirectorySearch;
+  private String trackingPolicy;
 
   @Override
   public synchronized void start() {
@@ -84,6 +92,7 @@ Configurable, EventDrivenSource {
       reader = new ReliableSpoolingFileEventReader.Builder()
           .spoolDirectory(directory)
           .completedSuffix(completedSuffix)
+          .includePattern(includePattern)
           .ignorePattern(ignorePattern)
           .trackerDirPath(trackerDirPath)
           .annotateFileName(fileHeader)
@@ -96,6 +105,9 @@ Configurable, EventDrivenSource {
           .inputCharset(inputCharset)
           .decodeErrorPolicy(decodeErrorPolicy)
           .consumeOrder(consumeOrder)
+          .recursiveDirectorySearch(recursiveDirectorySearch)
+          .trackingPolicy(trackingPolicy)
+          .sourceCounter(sourceCounter)
           .build();
     } catch (IOException ioe) {
       throw new FlumeException("Error instantiating spooling event parser",
@@ -104,7 +116,7 @@ Configurable, EventDrivenSource {
 
     Runnable runner = new SpoolDirectoryRunnable(reader, sourceCounter);
     executor.scheduleWithFixedDelay(
-        runner, 0, POLL_DELAY_MS, TimeUnit.MILLISECONDS);
+        runner, 0, pollDelay, TimeUnit.MILLISECONDS);
 
     super.start();
     logger.debug("SpoolDirectorySource source started");
@@ -123,8 +135,7 @@ Configurable, EventDrivenSource {
 
     super.stop();
     sourceCounter.stop();
-    logger.info("SpoolDir source {} stopped. Metrics: {}", getName(),
-      sourceCounter);
+    logger.info("SpoolDir source {} stopped. Metrics: {}", getName(), sourceCounter);
   }
 
   @Override
@@ -155,17 +166,23 @@ Configurable, EventDrivenSource {
     inputCharset = context.getString(INPUT_CHARSET, DEFAULT_INPUT_CHARSET);
     decodeErrorPolicy = DecodeErrorPolicy.valueOf(
         context.getString(DECODE_ERROR_POLICY, DEFAULT_DECODE_ERROR_POLICY)
-        .toUpperCase());
+            .toUpperCase(Locale.ENGLISH));
 
+    includePattern = context.getString(INCLUDE_PAT, DEFAULT_INCLUDE_PAT);
     ignorePattern = context.getString(IGNORE_PAT, DEFAULT_IGNORE_PAT);
     trackerDirPath = context.getString(TRACKER_DIR, DEFAULT_TRACKER_DIR);
 
     deserializerType = context.getString(DESERIALIZER, DEFAULT_DESERIALIZER);
     deserializerContext = new Context(context.getSubProperties(DESERIALIZER +
         "."));
-    
-    consumeOrder = ConsumeOrder.valueOf(context.getString(CONSUME_ORDER, 
-        DEFAULT_CONSUME_ORDER.toString()).toUpperCase());
+
+    consumeOrder = ConsumeOrder.valueOf(context.getString(CONSUME_ORDER,
+        DEFAULT_CONSUME_ORDER.toString()).toUpperCase(Locale.ENGLISH));
+
+    pollDelay = context.getInteger(POLL_DELAY, DEFAULT_POLL_DELAY);
+
+    recursiveDirectorySearch = context.getBoolean(RECURSIVE_DIRECTORY_SEARCH,
+        DEFAULT_RECURSIVE_DIRECTORY_SEARCH);
 
     // "Hack" to support backwards compatibility with previous generation of
     // spooling directory source, which did not support deserializers
@@ -180,6 +197,7 @@ Configurable, EventDrivenSource {
     if (sourceCounter == null) {
       sourceCounter = new SourceCounter(getName());
     }
+    trackingPolicy = context.getString(TRACKING_POLICY, DEFAULT_TRACKING_POLICY);
   }
 
   @VisibleForTesting
@@ -188,10 +206,10 @@ Configurable, EventDrivenSource {
   }
 
 
-
   /**
    * The class always backs off, this exists only so that we can test without
    * taking a really long time.
+   *
    * @param backoff - whether the source should backoff if the channel is full
    */
   @VisibleForTesting
@@ -200,21 +218,37 @@ Configurable, EventDrivenSource {
   }
 
   @VisibleForTesting
-  protected boolean hitChannelException() {
+  protected boolean didHitChannelException() {
     return hitChannelException;
   }
 
+  @VisibleForTesting
+  protected boolean didHitChannelFullException() {
+    return hitChannelFullException;
+  }
+
   @VisibleForTesting
   protected SourceCounter getSourceCounter() {
     return sourceCounter;
   }
 
-  private class SpoolDirectoryRunnable implements Runnable {
+  @VisibleForTesting
+  protected boolean getRecursiveDirectorySearch() {
+    return recursiveDirectorySearch;
+  }
+
+  @Override
+  public long getBatchSize() {
+    return batchSize;
+  }
+
+  @VisibleForTesting
+  protected class SpoolDirectoryRunnable implements Runnable {
     private ReliableSpoolingFileEventReader reader;
     private SourceCounter sourceCounter;
 
     public SpoolDirectoryRunnable(ReliableSpoolingFileEventReader reader,
-        SourceCounter sourceCounter) {
+                                  SourceCounter sourceCounter) {
       this.reader = reader;
       this.sourceCounter = sourceCounter;
     }
@@ -222,9 +256,12 @@ Configurable, EventDrivenSource {
     @Override
     public void run() {
       int backoffInterval = 250;
+      boolean readingEvents = false;
       try {
         while (!Thread.interrupted()) {
+          readingEvents = true;
           List<Event> events = reader.readEvents(batchSize);
+          readingEvents = false;
           if (events.isEmpty()) {
             break;
           }
@@ -234,31 +271,49 @@ Configurable, EventDrivenSource {
           try {
             getChannelProcessor().processEventBatch(events);
             reader.commit();
-          } catch (ChannelException ex) {
+          } catch (ChannelFullException ex) {
             logger.warn("The channel is full, and cannot write data now. The " +
-              "source will try again after " + String.valueOf(backoffInterval) +
-              " milliseconds");
+                "source will try again after " + backoffInterval +
+                " milliseconds");
+            sourceCounter.incrementChannelWriteFail();
+            hitChannelFullException = true;
+            backoffInterval = waitAndGetNewBackoffInterval(backoffInterval);
+            continue;
+          } catch (ChannelException ex) {
+            logger.warn("The channel threw an exception, and cannot write data now. The " +
+                "source will try again after " + backoffInterval +
+                " milliseconds");
+            sourceCounter.incrementChannelWriteFail();
             hitChannelException = true;
-            if (backoff) {
-              TimeUnit.MILLISECONDS.sleep(backoffInterval);
-              backoffInterval = backoffInterval << 1;
-              backoffInterval = backoffInterval >= maxBackoff ? maxBackoff :
-                                backoffInterval;
-            }
+            backoffInterval = waitAndGetNewBackoffInterval(backoffInterval);
             continue;
           }
           backoffInterval = 250;
           sourceCounter.addToEventAcceptedCount(events.size());
           sourceCounter.incrementAppendBatchAcceptedCount();
         }
-        logger.info("Spooling Directory Source runner has shutdown.");
       } catch (Throwable t) {
         logger.error("FATAL: " + SpoolDirectorySource.this.toString() + ": " +
             "Uncaught exception in SpoolDirectorySource thread. " +
             "Restart or reconfigure Flume to continue processing.", t);
+        if (readingEvents) {
+          sourceCounter.incrementEventReadFail();
+        } else {
+          sourceCounter.incrementGenericProcessingFail();
+        }
         hasFatalError = true;
         Throwables.propagate(t);
       }
     }
+
+    private int waitAndGetNewBackoffInterval(int backoffInterval) throws InterruptedException {
+      if (backoff) {
+        TimeUnit.MILLISECONDS.sleep(backoffInterval);
+        backoffInterval = backoffInterval << 1;
+        backoffInterval = backoffInterval >= maxBackoff ? maxBackoff :
+            backoffInterval;
+      }
+      return backoffInterval;
+    }
   }
 }
