@@ -23,7 +23,6 @@ import org.apache.flume.Event;
 import org.apache.flume.annotations.InterfaceAudience;
 import org.apache.flume.annotations.InterfaceStability;
 import org.apache.flume.event.EventBuilder;
-import org.jboss.netty.buffer.ChannelBuffer;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -33,6 +32,8 @@ import java.net.SocketAddress;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.time.Clock;
+import java.time.ZoneOffset;
+import java.time.temporal.ChronoField;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
@@ -46,6 +47,8 @@ import java.util.regex.MatchResult;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import io.netty.buffer.ByteBuf;
+
 @InterfaceAudience.Private
 @InterfaceStability.Evolving
 public class SyslogUtils {
@@ -175,10 +178,9 @@ public class SyslogUtils {
       if (keepFields.contains(SyslogSourceConfigurationConstants.CONFIG_KEEP_FIELDS_TIMESTAMP)) {
         body = timestamp + " " + body;
       }
-      if (keepFields.contains(SyslogSourceConfigurationConstants.CONFIG_KEEP_FIELDS_VERSION)) {
-        if (version != null && !version.isEmpty()) {
-          body = version + " " + body;
-        }
+      if (keepFields.contains(SyslogSourceConfigurationConstants.CONFIG_KEEP_FIELDS_VERSION)
+          && version != null && !version.isEmpty()) {
+        body = version + " " + body;
       }
       if (keepFields.contains(SyslogSourceConfigurationConstants.CONFIG_KEEP_FIELDS_PRIORITY)) {
         body = "<" + priority + ">" + body;
@@ -239,7 +241,7 @@ public class SyslogUtils {
     this.clock = clock;
     isBadEvent = false;
     isIncompleteEvent = false;
-    maxSize = (eventSize < MIN_SIZE) ? MIN_SIZE : eventSize;
+    maxSize = eventSize < MIN_SIZE ? MIN_SIZE : eventSize;
     baos = new ByteArrayOutputStream(eventSize);
     this.keepFields = keepFields;
     initHeaderFormats();
@@ -247,24 +249,19 @@ public class SyslogUtils {
 
   // extend the default header formatter
   public void addFormats(Map<String, String> formatProp) {
-    if (formatProp.isEmpty() || !formatProp.containsKey(
-        SyslogSourceConfigurationConstants.CONFIG_REGEX)) {
+    if (formatProp.isEmpty() || !formatProp.containsKey(SyslogSourceConfigurationConstants.CONFIG_REGEX)) {
       return;
     }
     SyslogFormatter fmt1 = new SyslogFormatter();
-    fmt1.regexPattern = Pattern.compile(
-        formatProp.get(SyslogSourceConfigurationConstants.CONFIG_REGEX));
+    fmt1.regexPattern = Pattern.compile(formatProp.get(SyslogSourceConfigurationConstants.CONFIG_REGEX));
     if (formatProp.containsKey(SyslogSourceConfigurationConstants.CONFIG_SEARCH)) {
-      fmt1.searchPattern.add(
-          formatProp.get(SyslogSourceConfigurationConstants.CONFIG_SEARCH));
+      fmt1.searchPattern.add(formatProp.get(SyslogSourceConfigurationConstants.CONFIG_SEARCH));
     }
     if (formatProp.containsKey(SyslogSourceConfigurationConstants.CONFIG_REPLACE)) {
-      fmt1.replacePattern.add(
-          formatProp.get(SyslogSourceConfigurationConstants.CONFIG_REPLACE));
+      fmt1.replacePattern.add(formatProp.get(SyslogSourceConfigurationConstants.CONFIG_REPLACE));
     }
     if (formatProp.containsKey(SyslogSourceConfigurationConstants.CONFIG_DATEFORMAT)) {
-      fmt1.dateFormat.add(new SimpleDateFormat(
-          formatProp.get(SyslogSourceConfigurationConstants.CONFIG_DATEFORMAT)));
+      fmt1.dateFormat.add(new SimpleDateFormat(formatProp.get(SyslogSourceConfigurationConstants.CONFIG_DATEFORMAT)));
     }
     formats.add(0, fmt1);
   }
@@ -306,8 +303,6 @@ public class SyslogUtils {
     START, PRIO, DATA
   }
 
-  ;
-
   public enum SyslogStatus {
     OTHER("Unknown"),
     INVALID("Invalid"),
@@ -315,7 +310,7 @@ public class SyslogUtils {
 
     private final String syslogStatus;
 
-    private SyslogStatus(String status) {
+    SyslogStatus(String status) {
       syslogStatus = status;
     }
 
@@ -342,16 +337,16 @@ public class SyslogUtils {
       Map<String, String> headers = new HashMap<String, String>();
       headers.put(SYSLOG_FACILITY, String.valueOf(facility));
       headers.put(SYSLOG_SEVERITY, String.valueOf(sev));
-      if ((priority != null) && (priority.length() > 0)) {
+      if (priority != null && priority.length() > 0) {
         headers.put("priority", priority);
       }
-      if ((version != null) && (version.length() > 0)) {
+      if (version != null && version.length() > 0) {
         headers.put("version", version);
       }
-      if ((timeStamp != null) && timeStamp.length() > 0) {
+      if (timeStamp != null && timeStamp.length() > 0) {
         headers.put("timestamp", timeStamp);
       }
-      if ((hostName != null) && (hostName.length() > 0)) {
+      if (hostName != null && hostName.length() > 0) {
         headers.put("host", hostName);
       }
       if (isBadEvent) {
@@ -364,7 +359,7 @@ public class SyslogUtils {
       }
 
       if (!keepAllFields(keepFields)) {
-        if ((msgBody != null) && (msgBody.length() > 0)) {
+        if (msgBody != null && msgBody.length() > 0) {
           body = msgBody.getBytes();
         } else {
           // Parse failed.
@@ -405,7 +400,7 @@ public class SyslogUtils {
             }
             // Add year to timestamp if needed
             if (fmt.addYear) {
-              value = String.valueOf(Calendar.getInstance().get(Calendar.YEAR)) + value;
+              value = clock.instant().atOffset(ZoneOffset.UTC).get(ChronoField.YEAR) + value;
             }
             // try the available time formats to timestamp
             for (int dt = 0; dt < fmt.dateFormat.size(); dt++) {
@@ -426,10 +421,6 @@ public class SyslogUtils {
                  * 1 month in the future) of timestamps.
                  */
                 if (fmt.addYear) {
-                  // Parsing from dateformatter without year part would use system clock
-                  // so we have to set the year part from the used clock instance
-                  parsedDate.setYear(new Date(clock.millis()).getYear());
-
                   Calendar calParsed = Calendar.getInstance();
                   calParsed.setTime(parsedDate);
                   Calendar calMinusOneMonth = Calendar.getInstance();
@@ -493,7 +484,7 @@ public class SyslogUtils {
   }
 
   // extract relevant syslog data needed for building Flume event
-  public Event extractEvent(ChannelBuffer in) {
+  public Event extractEvent(ByteBuf in) {
 
     /* for protocol debugging
     ByteBuffer bb = in.toByteBuffer();
@@ -507,76 +498,72 @@ public class SyslogUtils {
     Event e = null;
     boolean doneReading = false;
 
-    try {
-      while (!doneReading && in.readable()) {
-        b = in.readByte();
-        switch (m) {
-          case START:
-            if (b == '<') {
-              baos.write(b);
-              m = Mode.PRIO;
-            } else if (b == '\n') {
-              //If the character is \n, it was because the last event was exactly
-              //as long  as the maximum size allowed and
-              //the only remaining character was the delimiter - '\n', or
-              //multiple delimiters were sent in a row.
-              //Just ignore it, and move forward, don't change the mode.
-              //This is a no-op, just ignore it.
-              logger.debug("Delimiter found while in START mode, ignoring..");
-
-            } else {
-              isBadEvent = true;
-              baos.write(b);
-              //Bad event, just dump everything as if it is data.
-              m = Mode.DATA;
-            }
-            break;
-          case PRIO:
+    while (!doneReading && in.isReadable()) {
+      b = in.readByte();
+      switch (m) {
+        case START:
+          if (b == '<') {
             baos.write(b);
-            if (b == '>') {
-              if (prio.length() == 0) {
-                isBadEvent = true;
-              }
+            m = Mode.PRIO;
+          } else if (b == '\n') {
+            //If the character is \n, it was because the last event was exactly
+            //as long  as the maximum size allowed and
+            //the only remaining character was the delimiter - '\n', or
+            //multiple delimiters were sent in a row.
+            //Just ignore it, and move forward, don't change the mode.
+            //This is a no-op, just ignore it.
+            logger.debug("Delimiter found while in START mode, ignoring..");
+
+          } else {
+            isBadEvent = true;
+            baos.write(b);
+            //Bad event, just dump everything as if it is data.
+            m = Mode.DATA;
+          }
+          break;
+        case PRIO:
+          baos.write(b);
+          if (b == '>') {
+            if (prio.length() == 0) {
+              isBadEvent = true;
+            }
+            m = Mode.DATA;
+          } else {
+            char ch = (char) b;
+            prio.append(ch);
+            // Priority is max 3 digits per both RFC 3164 and 5424
+            // With this check there is basically no danger of
+            // boas.size() exceeding this.maxSize before getting to the
+            // DATA state where this is actually checked
+            if (!Character.isDigit(ch) || prio.length() > 3) {
+              isBadEvent = true;
+              //If we hit a bad priority, just write as if everything is data.
               m = Mode.DATA;
-            } else {
-              char ch = (char) b;
-              prio.append(ch);
-              // Priority is max 3 digits per both RFC 3164 and 5424
-              // With this check there is basically no danger of
-              // boas.size() exceeding this.maxSize before getting to the
-              // DATA state where this is actually checked
-              if (!Character.isDigit(ch) || prio.length() > 3) {
-                isBadEvent = true;
-                //If we hit a bad priority, just write as if everything is data.
-                m = Mode.DATA;
-              }
             }
-            break;
-          case DATA:
-            // TCP syslog entries are separated by '\n'
-            if (b == '\n') {
-              e = buildEvent();
-              doneReading = true;
-            } else {
-              baos.write(b);
-            }
-            if (baos.size() == this.maxSize && !doneReading) {
-              isIncompleteEvent = true;
-              e = buildEvent();
-              doneReading = true;
-            }
-            break;
-        }
-
+          }
+          break;
+        case DATA:
+          // TCP syslog entries are separated by '\n'
+          if (b == '\n') {
+            e = buildEvent();
+            doneReading = true;
+          } else {
+            baos.write(b);
+          }
+          if (baos.size() == this.maxSize && !doneReading) {
+            isIncompleteEvent = true;
+            e = buildEvent();
+            doneReading = true;
+          }
+          break;
       }
 
-      // UDP doesn't send a newline, so just use what we received
-      if (e == null && isUdp) {
-        doneReading = true;
-        e = buildEvent();
-      }
-    } finally {
-      // no-op
+    }
+
+    // UDP doesn't send a newline, so just use what we received
+    if (e == null && isUdp) {
+      doneReading = true;
+      e = buildEvent();
     }
 
     return e;
