@@ -25,13 +25,21 @@ import org.apache.flume.Event;
 import org.apache.flume.EventDrivenSource;
 import org.apache.flume.conf.Configurable;
 import org.apache.flume.instrumentation.SourceCounter;
-import org.apache.flume.source.AbstractSource;
+import org.apache.flume.source.SslContextAwareAbstractSource;
+import org.apache.flume.tools.FlumeBeanConfigurator;
 import org.apache.flume.tools.HTTPServerConstraintUtil;
-import org.mortbay.jetty.Connector;
-import org.mortbay.jetty.Server;
-import org.mortbay.jetty.nio.SelectChannelConnector;
-import org.mortbay.jetty.security.SslSocketConnector;
-import org.mortbay.jetty.servlet.ServletHolder;
+import org.eclipse.jetty.server.HttpConfiguration;
+import org.eclipse.jetty.server.HttpConnectionFactory;
+import org.eclipse.jetty.server.SecureRequestCustomizer;
+import org.eclipse.jetty.server.Server;
+import org.eclipse.jetty.server.ServerConnector;
+import org.eclipse.jetty.server.SslConnectionFactory;
+import org.eclipse.jetty.http.HttpVersion;
+import org.eclipse.jetty.jmx.MBeanContainer;
+import org.eclipse.jetty.servlet.ServletContextHandler;
+import org.eclipse.jetty.servlet.ServletHolder;
+import org.eclipse.jetty.util.ssl.SslContextFactory;
+import org.eclipse.jetty.util.thread.QueuedThreadPool;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -39,6 +47,7 @@ import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import java.io.IOException;
+import java.lang.management.ManagementFactory;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -67,7 +76,7 @@ import java.util.Map;
  * A JSON handler which converts JSON objects to Flume events is provided.
  *
  */
-public class HTTPSource extends AbstractSource implements
+public class HTTPSource extends SslContextAwareAbstractSource implements
         EventDrivenSource, Configurable {
   /*
    * There are 2 ways of doing this:
@@ -89,49 +98,32 @@ public class HTTPSource extends AbstractSource implements
   private HTTPSourceHandler handler;
   private SourceCounter sourceCounter;
 
-  // SSL configuration variable
-  private volatile String keyStorePath;
-  private volatile String keyStorePassword;
-  private volatile Boolean sslEnabled;
-
+  private Context sourceContext;
 
   @Override
   public void configure(Context context) {
+    configureSsl(context);
+    sourceContext = context;
     try {
-      // SSL related config
-      sslEnabled = context.getBoolean(HTTPSourceConfigurationConstants.SSL_ENABLED, false);
-
       port = context.getInteger(HTTPSourceConfigurationConstants.CONFIG_PORT);
       host = context.getString(HTTPSourceConfigurationConstants.CONFIG_BIND,
-        HTTPSourceConfigurationConstants.DEFAULT_BIND);
+          HTTPSourceConfigurationConstants.DEFAULT_BIND);
 
       Preconditions.checkState(host != null && !host.isEmpty(),
                 "HTTPSource hostname specified is empty");
       Preconditions.checkNotNull(port, "HTTPSource requires a port number to be"
-        + " specified");
+          + " specified");
 
       String handlerClassName = context.getString(
               HTTPSourceConfigurationConstants.CONFIG_HANDLER,
               HTTPSourceConfigurationConstants.DEFAULT_HANDLER).trim();
 
-      if(sslEnabled) {
-        LOG.debug("SSL configuration enabled");
-        keyStorePath = context.getString(HTTPSourceConfigurationConstants.SSL_KEYSTORE);
-        Preconditions.checkArgument(keyStorePath != null && !keyStorePath.isEmpty(),
-                                        "Keystore is required for SSL Conifguration" );
-        keyStorePassword = context.getString(HTTPSourceConfigurationConstants.SSL_KEYSTORE_PASSWORD);
-        Preconditions.checkArgument(keyStorePassword != null, "Keystore password is required for SSL Configuration");
-      }
-
-
-
       @SuppressWarnings("unchecked")
       Class<? extends HTTPSourceHandler> clazz =
               (Class<? extends HTTPSourceHandler>)
               Class.forName(handlerClassName);
       handler = clazz.getDeclaredConstructor().newInstance();
-      //ref: http://docs.codehaus.org/display/JETTY/Embedding+Jetty
-      //ref: http://jetty.codehaus.org/jetty/jetty-6/apidocs/org/mortbay/jetty/servlet/Context.html
+
       Map<String, String> subProps =
               context.getSubProperties(
               HTTPSourceConfigurationConstants.CONFIG_HANDLER_PREFIX);
@@ -152,48 +144,61 @@ public class HTTPSource extends AbstractSource implements
     }
   }
 
-  private void checkHostAndPort() {
-    Preconditions.checkState(host != null && !host.isEmpty(),
-      "HTTPSource hostname specified is empty");
-    Preconditions.checkNotNull(port, "HTTPSource requires a port number to be"
-      + " specified");
-  }
-
   @Override
   public void start() {
     Preconditions.checkState(srv == null,
             "Running HTTP Server found in source: " + getName()
             + " before I started one."
             + "Will not attempt to start.");
-    srv = new Server();
-
-    // Connector Array
-    Connector[] connectors = new Connector[1];
-
-
-    if (sslEnabled) {
-      SslSocketConnector sslSocketConnector = new SslSocketConnector();
-      sslSocketConnector.setKeystore(keyStorePath);
-      sslSocketConnector.setKeyPassword(keyStorePassword);
-      sslSocketConnector.setReuseAddress(true);
-      connectors[0] = sslSocketConnector;
-    } else {
-      SelectChannelConnector connector = new SelectChannelConnector();
-      connector.setReuseAddress(true);
-      connectors[0] = connector;
+    QueuedThreadPool threadPool = new QueuedThreadPool();
+    if (sourceContext.getSubProperties("QueuedThreadPool.").size() > 0) {
+      FlumeBeanConfigurator.setConfigurationFields(threadPool, sourceContext);
     }
+    srv = new Server(threadPool);
+
+    //Register with JMX for advanced monitoring
+    MBeanContainer mbContainer = new MBeanContainer(ManagementFactory.getPlatformMBeanServer());
+    srv.addEventListener(mbContainer);
+    srv.addBean(mbContainer);
+
+    HttpConfiguration httpConfiguration = new HttpConfiguration();
+    httpConfiguration.addCustomizer(new SecureRequestCustomizer());
+
+    FlumeBeanConfigurator.setConfigurationFields(httpConfiguration, sourceContext);
+    ServerConnector connector = getSslContextSupplier().get().map(sslContext -> {
+      SslContextFactory sslCtxFactory = new SslContextFactory();
+      sslCtxFactory.setSslContext(sslContext);
+      sslCtxFactory.setExcludeProtocols(getExcludeProtocols().toArray(new String[]{}));
+      sslCtxFactory.setIncludeProtocols(getIncludeProtocols().toArray(new String[]{}));
+      sslCtxFactory.setExcludeCipherSuites(getExcludeCipherSuites().toArray(new String[]{}));
+      sslCtxFactory.setIncludeCipherSuites(getIncludeCipherSuites().toArray(new String[]{}));
+
+      FlumeBeanConfigurator.setConfigurationFields(sslCtxFactory, sourceContext);
+
+      httpConfiguration.setSecurePort(port);
+      httpConfiguration.setSecureScheme("https");
+
+      return new ServerConnector(srv,
+          new SslConnectionFactory(sslCtxFactory, HttpVersion.HTTP_1_1.asString()),
+          new HttpConnectionFactory(httpConfiguration));
+    }).orElse(new ServerConnector(srv, new HttpConnectionFactory(httpConfiguration)));
+
+    connector.setPort(port);
+    connector.setHost(host);
+    connector.setReuseAddress(true);
+
+    FlumeBeanConfigurator.setConfigurationFields(connector, sourceContext);
+
+    srv.addConnector(connector);
 
-    connectors[0].setHost(host);
-    connectors[0].setPort(port);
-    srv.setConnectors(connectors);
     try {
-      org.mortbay.jetty.servlet.Context root =
-        new org.mortbay.jetty.servlet.Context(
-          srv, "/", org.mortbay.jetty.servlet.Context.SESSIONS);
-      root.addServlet(new ServletHolder(new FlumeHTTPServlet()), "/");
-      HTTPServerConstraintUtil.enforceConstraints(root);
+      ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
+      context.setContextPath("/");
+      srv.setHandler(context);
+
+      context.addServlet(new ServletHolder(new FlumeHTTPServlet()),"/");
+      context.setSecurityHandler(HTTPServerConstraintUtil.enforceConstraints());
       srv.start();
-      Preconditions.checkArgument(srv.getHandler().equals(root));
     } catch (Exception ex) {
       LOG.error("Error while starting HTTPSource. Exception follows.", ex);
       Throwables.propagate(ex);
@@ -228,12 +233,14 @@ public class HTTPSource extends AbstractSource implements
         events = handler.getEvents(request);
       } catch (HTTPBadRequestException ex) {
         LOG.warn("Received bad request from client. ", ex);
+        sourceCounter.incrementEventReadFail();
         response.sendError(HttpServletResponse.SC_BAD_REQUEST,
                 "Bad request from client. "
                 + ex.getMessage());
         return;
       } catch (Exception ex) {
         LOG.warn("Deserializer threw unexpected exception. ", ex);
+        sourceCounter.incrementEventReadFail();
         response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
                 "Deserializer threw unexpected exception. "
                 + ex.getMessage());
@@ -247,12 +254,14 @@ public class HTTPSource extends AbstractSource implements
         LOG.warn("Error appending event to channel. "
                 + "Channel might be full. Consider increasing the channel "
                 + "capacity or make sure the sinks perform faster.", ex);
+        sourceCounter.incrementChannelWriteFail();
         response.sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE,
                 "Error appending event to channel. Channel might be full."
                 + ex.getMessage());
         return;
       } catch (Exception ex) {
         LOG.warn("Unexpected error appending event to channel. ", ex);
+        sourceCounter.incrementGenericProcessingFail();
         response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
                 "Unexpected error while appending event to channel. "
                 + ex.getMessage());
@@ -271,4 +280,20 @@ public class HTTPSource extends AbstractSource implements
       doPost(request, response);
     }
   }
+
+  @Override
+  protected void configureSsl(Context context) {
+    handleDeprecatedParameter(context, "ssl", "enableSSL");
+    handleDeprecatedParameter(context, "exclude-protocols", "excludeProtocols");
+    handleDeprecatedParameter(context, "keystore-password", "keystorePassword");
+
+    super.configureSsl(context);
+  }
+
+  private void handleDeprecatedParameter(Context context, String newParam, String oldParam) {
+    if (!context.containsKey(newParam) && context.containsKey(oldParam)) {
+      context.put(newParam, context.getString(oldParam));
+    }
+  }
+
 }
