@@ -41,6 +41,7 @@ import org.apache.commons.io.FileUtils;
 import org.apache.commons.lang.SystemUtils;
 import org.apache.commons.lang.mutable.MutableLong;
 import org.apache.flume.Event;
+import org.apache.flume.FlumeException;
 import org.apache.flume.client.avro.ReliableSpoolingFileEventReader.DeletePolicy;
 import org.apache.flume.instrumentation.SourceCounter;
 import org.apache.flume.client.avro.ReliableSpoolingFileEventReader.TrackingPolicy;
@@ -74,7 +75,7 @@ public class TestReliableSpoolingFileEventReader {
     if (!WORK_DIR.isDirectory()) {
       Files.createParentDirs(new File(WORK_DIR, "dummy"));
     }
-
+    long lastModified = 0;
     // write out a few files
     for (int i = 0; i < 4; i++) {
       File fileName = new File(WORK_DIR, "file" + i);
@@ -85,6 +86,12 @@ public class TestReliableSpoolingFileEventReader {
         sb.append("file" + i + "line" + j + "\n");
       }
       Files.write(sb.toString(), fileName, Charsets.UTF_8);
+      // Make sure all the files have the same timestamp.
+      if (lastModified == 0) {
+        lastModified = fileName.lastModified();
+      } else {
+        fileName.setLastModified(lastModified);
+      }
     }
     Thread.sleep(1500L); // make sure timestamp is later
     Files.write("\n", new File(WORK_DIR, "emptylineFile"), Charsets.UTF_8);
@@ -92,9 +99,15 @@ public class TestReliableSpoolingFileEventReader {
 
   @After
   public void tearDown() {
+    setDirWritable(WORK_DIR);
     deleteDir(WORK_DIR);
   }
 
+  private void setDirWritable(File dir){
+    // make dir writable
+    dir.setWritable(true);
+  }
+
   private void deleteDir(File dir) {
     // delete all the files & dirs we created
     try {
@@ -315,6 +328,41 @@ public class TestReliableSpoolingFileEventReader {
         trackerFiles.size());
   }
 
+  @Test(expected = FlumeException.class)
+  public void testRenameTrackingPolicyOnReadonlySpoolDirectory() throws IOException {
+    File workDir = WORK_DIR;
+    if(workDir.setReadOnly()){
+      new ReliableSpoolingFileEventReader.Builder().spoolDirectory(workDir)
+              .trackingPolicy(TrackingPolicy.RENAME.name())
+              .sourceCounter(new SourceCounter("test"))
+              .build();
+    } else {
+      // Operation on directory permission is not supported in current operating system.
+      throw new FlumeException("Operation on directory permission is not supported in current operating system.");
+    }
+  }
+
+  @Test()
+  public void testTrackerDirTrackingPolicyOnReadonlySpoolDirectory() throws IOException {
+    File workDir = WORK_DIR;
+    String trackerDirPath =
+            SpoolDirectorySourceConfigurationConstants.DEFAULT_TRACKER_DIR;
+    File trackerDir = new File(WORK_DIR, trackerDirPath);
+    if (!trackerDir.exists()) {
+      trackerDir.mkdir();
+    }
+    if(workDir.setReadOnly()){
+      new ReliableSpoolingFileEventReader.Builder().spoolDirectory(workDir)
+              .trackingPolicy(TrackingPolicy.TRACKER_DIR.name())
+              .trackerDirPath(trackerDirPath)
+              .sourceCounter(new SourceCounter("test"))
+              .build();
+    } else {
+      // Operation on directory permission is not supported in current operating system.
+      return;
+    }
+  }
+
   @Test(expected = NullPointerException.class)
   public void testNullConsumeOrder() throws IOException {
     new ReliableSpoolingFileEventReader.Builder().spoolDirectory(WORK_DIR)
@@ -456,7 +504,7 @@ public class TestReliableSpoolingFileEventReader {
     FileUtils.write(file3, "New file3 created.\n");
     FileUtils.write(file2, "New file2 created.\n");
     FileUtils.write(file1, "New file1 created.\n");
-    file1.setLastModified(file3.lastModified());
+    file2.setLastModified(file3.lastModified());
     file1.setLastModified(file2.lastModified());
     // file ages are same now they need to be ordered
     // lexicographically (file1, file2, file3).
@@ -486,7 +534,7 @@ public class TestReliableSpoolingFileEventReader {
     FileUtils.write(file1, "New file1 created.\n");
     FileUtils.write(file2, "New file2 created.\n");
     FileUtils.write(file3, "New file3 created.\n");
-    file1.setLastModified(file3.lastModified());
+    file2.setLastModified(file3.lastModified());
     file1.setLastModified(file2.lastModified());
     // file ages are same now they need to be ordered
     // lexicographically (file1, file2, file3).
@@ -657,6 +705,10 @@ public class TestReliableSpoolingFileEventReader {
     for (int i = 0; i < listFiles(dir).size(); i++) {
       events = reader.readEvents(10);
       for (Event e : events) {
+        if (reader instanceof ReliableSpoolingFileEventReader) {
+          logger.debug("Adding event for file {} with body \"{}\"",
+              ((ReliableSpoolingFileEventReader) reader).getLastFileReadInfo(), new String(e.getBody()));
+        }
         actual.add(new String(e.getBody()));
       }
       reader.commit();
