@@ -17,8 +17,17 @@
  */
 package org.apache.flume.node;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Properties;
+import java.util.Set;
 
 import org.apache.flume.Channel;
 import org.apache.flume.ChannelFactory;
@@ -37,10 +46,12 @@ import org.apache.flume.channel.ChannelProcessor;
 import org.apache.flume.channel.ChannelSelectorFactory;
 import org.apache.flume.channel.DefaultChannelFactory;
 import org.apache.flume.conf.BasicConfigurationConstants;
+import org.apache.flume.conf.BatchSizeSupported;
 import org.apache.flume.conf.ComponentConfiguration;
 import org.apache.flume.conf.Configurables;
 import org.apache.flume.conf.FlumeConfiguration;
 import org.apache.flume.conf.FlumeConfiguration.AgentConfiguration;
+import org.apache.flume.conf.TransactionCapacitySupported;
 import org.apache.flume.conf.channel.ChannelSelectorConfiguration;
 import org.apache.flume.conf.sink.SinkConfiguration;
 import org.apache.flume.conf.sink.SinkGroupConfiguration;
@@ -58,18 +69,18 @@ import com.google.common.collect.ListMultimap;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 
-public abstract class AbstractConfigurationProvider implements
-    ConfigurationProvider {
+import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 
-  private static final Logger LOGGER = LoggerFactory
-      .getLogger(AbstractConfigurationProvider.class);
+@SuppressFBWarnings("REC_CATCH_EXCEPTION")
+public abstract class AbstractConfigurationProvider implements ConfigurationProvider {
+
+  private static final Logger LOGGER = LoggerFactory.getLogger(AbstractConfigurationProvider.class);
 
   private final String agentName;
   private final SourceFactory sourceFactory;
   private final SinkFactory sinkFactory;
   private final ChannelFactory channelFactory;
 
-
   private final Map<Class<? extends Channel>, Map<String, Channel>> channelCache;
 
   public AbstractConfigurationProvider(String agentName) {
@@ -96,18 +107,16 @@ public abstract class AbstractConfigurationProvider implements
         loadChannels(agentConf, channelComponentMap);
         loadSources(agentConf, channelComponentMap, sourceRunnerMap);
         loadSinks(agentConf, channelComponentMap, sinkRunnerMap);
-        Set<String> channelNames =
-            new HashSet<String>(channelComponentMap.keySet());
-        for(String channelName : channelNames) {
-          ChannelComponent channelComponent = channelComponentMap.
-              get(channelName);
-          if(channelComponent.components.isEmpty()) {
+        Set<String> channelNames = new HashSet<String>(channelComponentMap.keySet());
+        for (String channelName : channelNames) {
+          ChannelComponent channelComponent = channelComponentMap.get(channelName);
+          if (channelComponent.components.isEmpty()) {
             LOGGER.warn(String.format("Channel %s has no components connected" +
                 " and has been removed.", channelName));
             channelComponentMap.remove(channelName);
-            Map<String, Channel> nameChannelMap = channelCache.
-                get(channelComponent.channel.getClass());
-            if(nameChannelMap != null) {
+            Map<String, Channel> nameChannelMap =
+                channelCache.get(channelComponent.channel.getClass());
+            if (nameChannelMap != null) {
               nameChannelMap.remove(channelName);
             }
           } else {
@@ -116,10 +125,10 @@ public abstract class AbstractConfigurationProvider implements
             conf.addChannel(channelName, channelComponent.channel);
           }
         }
-        for(Map.Entry<String, SourceRunner> entry : sourceRunnerMap.entrySet()) {
+        for (Map.Entry<String, SourceRunner> entry : sourceRunnerMap.entrySet()) {
           conf.addSourceRunner(entry.getKey(), entry.getValue());
         }
-        for(Map.Entry<String, SinkRunner> entry : sinkRunnerMap.entrySet()) {
+        for (Map.Entry<String, SinkRunner> entry : sinkRunnerMap.entrySet()) {
           conf.addSinkRunner(entry.getKey(), entry.getValue());
         }
       } catch (InstantiationException ex) {
@@ -155,21 +164,21 @@ public abstract class AbstractConfigurationProvider implements
     ListMultimap<Class<? extends Channel>, String> channelsNotReused =
         ArrayListMultimap.create();
     // assume all channels will not be re-used
-    for(Map.Entry<Class<? extends Channel>, Map<String, Channel>> entry : channelCache.entrySet()) {
+    for (Map.Entry<Class<? extends Channel>, Map<String, Channel>> entry :
+         channelCache.entrySet()) {
       Class<? extends Channel> channelKlass = entry.getKey();
       Set<String> channelNames = entry.getValue().keySet();
       channelsNotReused.get(channelKlass).addAll(channelNames);
     }
 
     Set<String> channelNames = agentConf.getChannelSet();
-    Map<String, ComponentConfiguration> compMap =
-        agentConf.getChannelConfigMap();
+    Map<String, ComponentConfiguration> compMap = agentConf.getChannelConfigMap();
     /*
      * Components which have a ComponentConfiguration object
      */
     for (String chName : channelNames) {
       ComponentConfiguration comp = compMap.get(chName);
-      if(comp != null) {
+      if (comp != null) {
         Channel channel = getOrCreateChannel(channelsNotReused,
             comp.getComponentName(), comp.getType());
         try {
@@ -190,17 +199,16 @@ public abstract class AbstractConfigurationProvider implements
      */
     for (String chName : channelNames) {
       Context context = agentConf.getChannelContext().get(chName);
-      if(context != null){
-        Channel channel =
-            getOrCreateChannel(channelsNotReused, chName, context.getString(
-                BasicConfigurationConstants.CONFIG_TYPE));
+      if (context != null) {
+        Channel channel = getOrCreateChannel(channelsNotReused, chName,
+            context.getString(BasicConfigurationConstants.CONFIG_TYPE));
         try {
           Configurables.configure(channel, context);
           channelComponentMap.put(chName, new ChannelComponent(channel));
           LOGGER.info("Created channel " + chName);
         } catch (Exception e) {
           String msg = String.format("Channel %s has been removed due to an " +
-                "error during configuration", chName);
+              "error during configuration", chName);
           LOGGER.error(msg, e);
         }
       }
@@ -212,7 +220,7 @@ public abstract class AbstractConfigurationProvider implements
       Map<String, Channel> channelMap = channelCache.get(channelKlass);
       if (channelMap != null) {
         for (String channelName : channelsNotReused.get(channelKlass)) {
-          if(channelMap.remove(channelName) != null) {
+          if (channelMap.remove(channelName) != null) {
             LOGGER.info("Removed {} of type {}", channelName, channelKlass);
           }
         }
@@ -228,12 +236,11 @@ public abstract class AbstractConfigurationProvider implements
       String name, String type)
       throws FlumeException {
 
-    Class<? extends Channel> channelClass = channelFactory.
-        getClass(type);
+    Class<? extends Channel> channelClass = channelFactory.getClass(type);
     /*
      * Channel has requested a new instance on each re-configuration
      */
-    if(channelClass.isAnnotationPresent(Disposable.class)) {
+    if (channelClass.isAnnotationPresent(Disposable.class)) {
       Channel channel = channelFactory.create(name, type);
       channel.setName(name);
       return channel;
@@ -244,7 +251,7 @@ public abstract class AbstractConfigurationProvider implements
       channelCache.put(channelClass, channelMap);
     }
     Channel channel = channelMap.get(name);
-    if(channel == null) {
+    if (channel == null) {
       channel = channelFactory.create(name, type);
       channel.setName(name);
       channelMap.put(name, channel);
@@ -266,7 +273,7 @@ public abstract class AbstractConfigurationProvider implements
      */
     for (String sourceName : sourceNames) {
       ComponentConfiguration comp = compMap.get(sourceName);
-      if(comp != null) {
+      if (comp != null) {
         SourceConfiguration config = (SourceConfiguration) comp;
 
         Source source = sourceFactory.create(comp.getComponentName(),
@@ -274,14 +281,9 @@ public abstract class AbstractConfigurationProvider implements
         try {
           Configurables.configure(source, config);
           Set<String> channelNames = config.getChannels();
-          List<Channel> sourceChannels = new ArrayList<Channel>();
-          for (String chName : channelNames) {
-            ChannelComponent channelComponent = channelComponentMap.get(chName);
-            if(channelComponent != null) {
-              sourceChannels.add(channelComponent.channel);
-            }
-          }
-          if(sourceChannels.isEmpty()) {
+          List<Channel> sourceChannels =
+                  getSourceChannels(channelComponentMap, source, channelNames);
+          if (sourceChannels.isEmpty()) {
             String msg = String.format("Source %s is not connected to a " +
                 "channel",  sourceName);
             throw new IllegalStateException(msg);
@@ -298,10 +300,10 @@ public abstract class AbstractConfigurationProvider implements
           source.setChannelProcessor(channelProcessor);
           sourceRunnerMap.put(comp.getComponentName(),
               SourceRunner.forSource(source));
-          for(Channel channel : sourceChannels) {
-            ChannelComponent channelComponent = Preconditions.
-                checkNotNull(channelComponentMap.get(channel.getName()),
-                    String.format("Channel %s", channel.getName()));
+          for (Channel channel : sourceChannels) {
+            ChannelComponent channelComponent =
+                Preconditions.checkNotNull(channelComponentMap.get(channel.getName()),
+                                           String.format("Channel %s", channel.getName()));
             channelComponent.components.add(sourceName);
           }
         } catch (Exception e) {
@@ -318,22 +320,17 @@ public abstract class AbstractConfigurationProvider implements
     Map<String, Context> sourceContexts = agentConf.getSourceContext();
     for (String sourceName : sourceNames) {
       Context context = sourceContexts.get(sourceName);
-      if(context != null){
+      if (context != null) {
         Source source =
             sourceFactory.create(sourceName,
-                context.getString(BasicConfigurationConstants.CONFIG_TYPE));
+                                 context.getString(BasicConfigurationConstants.CONFIG_TYPE));
         try {
           Configurables.configure(source, context);
-          List<Channel> sourceChannels = new ArrayList<Channel>();
           String[] channelNames = context.getString(
               BasicConfigurationConstants.CONFIG_CHANNELS).split("\\s+");
-          for (String chName : channelNames) {
-            ChannelComponent channelComponent = channelComponentMap.get(chName);
-            if(channelComponent != null) {
-              sourceChannels.add(channelComponent.channel);
-            }
-          }
-          if(sourceChannels.isEmpty()) {
+          List<Channel> sourceChannels =
+                  getSourceChannels(channelComponentMap, source, Arrays.asList(channelNames));
+          if (sourceChannels.isEmpty()) {
             String msg = String.format("Source %s is not connected to a " +
                 "channel",  sourceName);
             throw new IllegalStateException(msg);
@@ -349,10 +346,10 @@ public abstract class AbstractConfigurationProvider implements
           source.setChannelProcessor(channelProcessor);
           sourceRunnerMap.put(sourceName,
               SourceRunner.forSource(source));
-          for(Channel channel : sourceChannels) {
-            ChannelComponent channelComponent = Preconditions.
-                checkNotNull(channelComponentMap.get(channel.getName()),
-                    String.format("Channel %s", channel.getName()));
+          for (Channel channel : sourceChannels) {
+            ChannelComponent channelComponent =
+                Preconditions.checkNotNull(channelComponentMap.get(channel.getName()),
+                                           String.format("Channel %s", channel.getName()));
             channelComponent.components.add(sourceName);
           }
         } catch (Exception e) {
@@ -364,6 +361,53 @@ public abstract class AbstractConfigurationProvider implements
     }
   }
 
+  private List<Channel> getSourceChannels(Map<String, ChannelComponent> channelComponentMap,
+                  Source source, Collection<String> channelNames) throws InstantiationException {
+    List<Channel> sourceChannels = new ArrayList<Channel>();
+    for (String chName : channelNames) {
+      ChannelComponent channelComponent = channelComponentMap.get(chName);
+      if (channelComponent != null) {
+        checkSourceChannelCompatibility(source, channelComponent.channel);
+        sourceChannels.add(channelComponent.channel);
+      }
+    }
+    return sourceChannels;
+  }
+
+  private void checkSourceChannelCompatibility(Source source, Channel channel)
+      throws InstantiationException {
+    if (source instanceof BatchSizeSupported && channel instanceof TransactionCapacitySupported) {
+      long transCap = ((TransactionCapacitySupported) channel).getTransactionCapacity();
+      long batchSize = ((BatchSizeSupported) source).getBatchSize();
+      if (transCap < batchSize) {
+        String msg = String.format(
+            "Incompatible source and channel settings defined. " +
+                "source's batch size is greater than the channels transaction capacity. " +
+                "Source: %s, batch size = %d, channel %s, transaction capacity = %d",
+            source.getName(), batchSize,
+            channel.getName(), transCap);
+        throw new InstantiationException(msg);
+      }
+    }
+  }
+
+  private void checkSinkChannelCompatibility(Sink sink, Channel channel)
+      throws InstantiationException {
+    if (sink instanceof BatchSizeSupported && channel instanceof TransactionCapacitySupported) {
+      long transCap = ((TransactionCapacitySupported) channel).getTransactionCapacity();
+      long batchSize = ((BatchSizeSupported) sink).getBatchSize();
+      if (transCap < batchSize) {
+        String msg = String.format(
+            "Incompatible sink and channel settings defined. " +
+                "sink's batch size is greater than the channels transaction capacity. " +
+                "Sink: %s, batch size = %d, channel %s, transaction capacity = %d",
+            sink.getName(), batchSize,
+            channel.getName(), transCap);
+        throw new InstantiationException(msg);
+      }
+    }
+  }
+
   private void loadSinks(AgentConfiguration agentConf,
       Map<String, ChannelComponent> channelComponentMap, Map<String, SinkRunner> sinkRunnerMap)
       throws InstantiationException {
@@ -376,19 +420,18 @@ public abstract class AbstractConfigurationProvider implements
      */
     for (String sinkName : sinkNames) {
       ComponentConfiguration comp = compMap.get(sinkName);
-      if(comp != null) {
+      if (comp != null) {
         SinkConfiguration config = (SinkConfiguration) comp;
-        Sink sink = sinkFactory.create(comp.getComponentName(),
-            comp.getType());
+        Sink sink = sinkFactory.create(comp.getComponentName(), comp.getType());
         try {
           Configurables.configure(sink, config);
-          ChannelComponent channelComponent = channelComponentMap.
-              get(config.getChannel());
-          if(channelComponent == null) {
+          ChannelComponent channelComponent = channelComponentMap.get(config.getChannel());
+          if (channelComponent == null) {
             String msg = String.format("Sink %s is not connected to a " +
                 "channel",  sinkName);
             throw new IllegalStateException(msg);
           }
+          checkSinkChannelCompatibility(sink, channelComponent.channel);
           sink.setChannel(channelComponent.channel);
           sinks.put(comp.getComponentName(), sink);
           channelComponent.components.add(sinkName);
@@ -406,18 +449,20 @@ public abstract class AbstractConfigurationProvider implements
     Map<String, Context> sinkContexts = agentConf.getSinkContext();
     for (String sinkName : sinkNames) {
       Context context = sinkContexts.get(sinkName);
-      if(context != null) {
+      if (context != null) {
         Sink sink = sinkFactory.create(sinkName, context.getString(
             BasicConfigurationConstants.CONFIG_TYPE));
         try {
           Configurables.configure(sink, context);
-          ChannelComponent channelComponent = channelComponentMap.
-              get(context.getString(BasicConfigurationConstants.CONFIG_CHANNEL));
-          if(channelComponent == null) {
+          ChannelComponent channelComponent =
+              channelComponentMap.get(
+                  context.getString(BasicConfigurationConstants.CONFIG_CHANNEL));
+          if (channelComponent == null) {
             String msg = String.format("Sink %s is not connected to a " +
                 "channel",  sinkName);
             throw new IllegalStateException(msg);
           }
+          checkSinkChannelCompatibility(sink, channelComponent.channel);
           sink.setChannel(channelComponent.channel);
           sinks.put(sinkName, sink);
           channelComponent.components.add(sinkName);
@@ -441,7 +486,7 @@ public abstract class AbstractConfigurationProvider implements
     Map<String, String> usedSinks = new HashMap<String, String>();
     for (String groupName: sinkGroupNames) {
       ComponentConfiguration comp = compMap.get(groupName);
-      if(comp != null) {
+      if (comp != null) {
         SinkGroupConfiguration groupConf = (SinkGroupConfiguration) comp;
         List<Sink> groupSinks = new ArrayList<Sink>();
         for (String sink : groupConf.getSinks()) {
@@ -475,7 +520,7 @@ public abstract class AbstractConfigurationProvider implements
       }
     }
     // add any unassigned sinks to solo collectors
-    for(Entry<String, Sink> entry : sinks.entrySet()) {
+    for (Entry<String, Sink> entry : sinks.entrySet()) {
       if (!usedSinks.containsValue(entry.getKey())) {
         try {
           SinkProcessor pr = new DefaultSinkProcessor();
@@ -483,9 +528,8 @@ public abstract class AbstractConfigurationProvider implements
           sinkMap.add(entry.getValue());
           pr.setSinks(sinkMap);
           Configurables.configure(pr, new Context());
-          sinkRunnerMap.put(entry.getKey(),
-              new SinkRunner(pr));
-        } catch(Exception e) {
+          sinkRunnerMap.put(entry.getKey(), new SinkRunner(pr));
+        } catch (Exception e) {
           String msg = String.format("SinkGroup %s has been removed due to " +
               "an error during configuration", entry.getKey());
           LOGGER.error(msg, e);
@@ -496,6 +540,7 @@ public abstract class AbstractConfigurationProvider implements
   private static class ChannelComponent {
     final Channel channel;
     final List<String> components;
+
     ChannelComponent(Channel channel) {
       this.channel = channel;
       components = Lists.newArrayList();
