@@ -17,16 +17,9 @@
  */
 package org.apache.flume.node;
 
-import java.io.BufferedReader;
 import java.io.File;
-import java.io.FileReader;
-import java.io.IOException;
-import java.util.HashMap;
-import java.util.Properties;
 
-import org.apache.flume.conf.FlumeConfiguration;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
+import com.google.common.collect.Lists;
 
 /**
  * <p>
@@ -96,7 +89,7 @@ import org.slf4j.LoggerFactory;
  * belonging to it. These cannot be shared by multiple groups.
  * Further, one can set a processor and behavioral parameters to determine
  * how sink selection is made via <tt>&lt;agent name&gt;.sinkgroups.&lt;
- * group name&lt.processor</tt>. For further detail refer to inividual processor
+ * group name&lt.processor</tt>. For further detail refer to individual processor
  * documentation</li>
  * <li>Sinks not assigned to a group will be assigned to default single sink
  * groups.</li>
@@ -165,41 +158,13 @@ import org.slf4j.LoggerFactory;
  * </p>
  *
  * @see java.util.Properties#load(java.io.Reader)
+ * @deprecated Use UriConfigurationProvider.
  */
-public class PropertiesFileConfigurationProvider extends
-    AbstractConfigurationProvider {
-
-  private static final Logger LOGGER = LoggerFactory
-      .getLogger(PropertiesFileConfigurationProvider.class);
-
-  private final File file;
+@Deprecated
+public class PropertiesFileConfigurationProvider extends UriConfigurationProvider {
 
   public PropertiesFileConfigurationProvider(String agentName, File file) {
-    super(agentName);
-    this.file = file;
-  }
-
-  @Override
-  public FlumeConfiguration getFlumeConfiguration() {
-    BufferedReader reader = null;
-    try {
-      reader = new BufferedReader(new FileReader(file));
-      Properties properties = new Properties();
-      properties.load(reader);
-      return new FlumeConfiguration(toMap(properties));
-    } catch (IOException ex) {
-      LOGGER.error("Unable to load file:" + file
-          + " (I/O failure) - Exception follows.", ex);
-    } finally {
-      if (reader != null) {
-        try {
-          reader.close();
-        } catch (IOException ex) {
-          LOGGER.warn(
-              "Unable to close file reader for file: " + file, ex);
-        }
-      }
-    }
-    return new FlumeConfiguration(new HashMap<String, String>());
+    super(agentName, Lists.newArrayList(new FileConfigurationSource(file.toURI())), null, null, 0);
+    super.start();
   }
 }
