@@ -17,21 +17,20 @@
 + */
 package org.apache.flume.sink.hdfs;
 
-import java.io.IOException;
-
 import org.apache.hadoop.fs.FSDataOutputStream;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-public class MockFsDataOutputStream extends FSDataOutputStream{
+import java.io.IOException;
+
+public class MockFsDataOutputStream extends FSDataOutputStream {
 
   private static final Logger logger =
       LoggerFactory.getLogger(MockFsDataOutputStream.class);
 
   boolean closeSucceed;
 
-  public MockFsDataOutputStream(FSDataOutputStream wrapMe,
-    boolean closeSucceed)
+  public MockFsDataOutputStream(FSDataOutputStream wrapMe, boolean closeSucceed)
       throws IOException {
     super(wrapMe.getWrappedStream(), null);
     this.closeSucceed = closeSucceed;
@@ -39,8 +38,7 @@ public class MockFsDataOutputStream extends FSDataOutputStream{
 
   @Override
   public void close() throws IOException {
-    logger.info(
-      "Close Succeeded - " + closeSucceed);
+    logger.info("Close Succeeded - " + closeSucceed);
     if (closeSucceed) {
       logger.info("closing file");
       super.close();
