@@ -64,6 +64,7 @@ public class TestJMSSource extends JMSMessageConsumerTestBase {
   @SuppressWarnings("unchecked")
   @Override
   void afterSetup() throws Exception {
+    System.setProperty(JMSSource.JNDI_ALLOWED_PROTOCOLS, "dummy");
     baseDir = Files.createTempDir();
     passwordFile = new File(baseDir, "password");
     Assert.assertTrue(passwordFile.createNewFile());
@@ -144,6 +145,13 @@ public class TestJMSSource extends JMSMessageConsumerTestBase {
     source.configure(context);
   }
 
+  @Test(expected = IllegalArgumentException.class)
+  public void testConfigureWithLdapProvider() throws Exception {
+    context.put(JMSSourceConfiguration.PROVIDER_URL, "ldap://localhost:389/test");
+    source.configure(context);
+  }
+
+
   @Test
   public void testStartConsumerCreateThrowsException() throws Exception {
     doThrow(new RuntimeException("Expected")).when(source).createConsumer();
