@@ -18,6 +18,15 @@
  */
 package org.apache.flume.test.util;
 
+import com.google.common.base.Joiner;
+import com.google.common.base.Preconditions;
+import com.google.common.collect.ImmutableList;
+import com.google.common.io.Files;
+import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
+import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 import java.io.File;
 import java.io.FileFilter;
 import java.io.FileInputStream;
@@ -25,30 +34,21 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.net.ServerSocket;
 import java.net.Socket;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
-import java.util.concurrent.TimeUnit;
 import java.util.zip.GZIPInputStream;
 
-import com.google.common.base.Preconditions;
-import com.google.common.io.Files;
-import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
-import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
-import org.apache.log4j.Logger;
-
-import com.google.common.base.Joiner;
-import com.google.common.collect.ImmutableList;
-
-
 /**
  * Attempts to setup a staged install using explicitly specified tar-ball
  * distribution or by using relative path into the flume-ng-dist module.
  */
 public class StagedInstall {
 
-  private static final Logger LOGGER = Logger.getLogger(StagedInstall.class);
+  private static final Logger LOGGER = LoggerFactory.getLogger(StagedInstall.class);
 
   public static final String PROP_PATH_TO_DIST_TARBALL =
       "flume.dist.tarball";
@@ -73,13 +73,19 @@ public class StagedInstall {
 
   private static StagedInstall INSTANCE;
 
-  public synchronized static StagedInstall getInstance() throws Exception {
+  public static synchronized StagedInstall getInstance() throws Exception {
     if (INSTANCE == null) {
       INSTANCE = new StagedInstall();
     }
     return INSTANCE;
   }
 
+  public static int findFreePort() throws IOException {
+    try (ServerSocket socket = new ServerSocket(0)) {
+      return socket.getLocalPort();
+    }
+  }
+
   public synchronized boolean isRunning() {
     return process != null;
   }
@@ -102,19 +108,26 @@ public class StagedInstall {
     Thread.sleep(3000); // sleep for 3s to let system shutdown
   }
 
-  public synchronized void startAgent(String name, String configResource)
+  public synchronized int startAgent(String name, String configResource)
       throws Exception {
     if (process != null) {
       throw new Exception("A process is already running");
     }
-
+    int port = findFreePort();
     Properties props = new Properties();
     props.load(ClassLoader.getSystemResourceAsStream(configResource));
-
+    props.put("rpccagent.sources.src1.port", String.valueOf(port));
     startAgent(name, props);
+    return port;
   }
 
-  public synchronized void startAgent(String name, Properties properties)
+  public synchronized void startAgent(String name, Properties properties) throws Exception {
+    startAgent(name, properties, new HashMap<>(), new HashMap<>());
+  }
+
+  public synchronized void startAgent(
+      String name, Properties properties,  Map<String, String> environmentVariables,
+      Map<String, String> commandOptions)
       throws Exception {
     Preconditions.checkArgument(!name.isEmpty(), "agent name must not be empty");
     Preconditions.checkNotNull(properties, "properties object must not be null");
@@ -124,8 +137,7 @@ public class StagedInstall {
     if (process != null) {
       throw new Exception("A process is already running");
     }
-    LOGGER.info("Starting process for agent: " + agentName + " using config: "
-       + properties);
+    LOGGER.info("Starting process for agent: " + agentName + " using config: " + properties);
 
     File configFile = createConfigurationFile(agentName, properties);
     configFilePath = configFile.getCanonicalPath();
@@ -147,6 +159,8 @@ public class StagedInstall {
             + ENV_FLUME_ROOT_LOGGER_VALUE);
     builder.add("-D" + ENV_FLUME_LOG_FILE + "=" + logFileName);
 
+    commandOptions.forEach((key, value) -> builder.add(key, value));
+
     List<String> cmdArgs = builder.build();
 
     LOGGER.info("Using command: " + Joiner.on(" ").join(cmdArgs));
@@ -154,6 +168,7 @@ public class StagedInstall {
     ProcessBuilder pb = new ProcessBuilder(cmdArgs);
 
     Map<String, String> env = pb.environment();
+    env.putAll(environmentVariables);
 
     LOGGER.debug("process environment: " + env);
 
@@ -252,7 +267,7 @@ public class StagedInstall {
     File[] listBaseDirs = stageDir.listFiles();
     if (listBaseDirs != null && listBaseDirs.length == 1
         && listBaseDirs[0].isDirectory()) {
-      rootDir =listBaseDirs[0];
+      rootDir = listBaseDirs[0];
     }
     baseDir = rootDir;
 
@@ -417,7 +432,6 @@ public class StagedInstall {
       if (testFile.exists() && testFile.isDirectory()) {
         LOGGER.info("Found candidate dir: " + testFile.getCanonicalPath());
         File[] candidateFiles = testFile.listFiles(new FileFilter() {
-
           @Override
           public boolean accept(File pathname) {
             String name = pathname.getName();
@@ -426,7 +440,8 @@ public class StagedInstall {
               return true;
             }
             return false;
-          }});
+          }
+        });
 
         // There should be at most one
         if (candidateFiles != null && candidateFiles.length > 0) {
@@ -441,7 +456,7 @@ public class StagedInstall {
               LOGGER.warn("Invalid file: " + file.getCanonicalPath());
             }
           } else {
-            StringBuilder sb = new StringBuilder("Multiple candate tarballs");
+            StringBuilder sb = new StringBuilder("Multiple candidate tarballs");
             sb.append(" found in directory ");
             sb.append(testFile.getCanonicalPath()).append(": ");
             boolean first = true;
@@ -466,22 +481,22 @@ public class StagedInstall {
   }
 
   public static void waitUntilPortOpens(String host, int port, long timeout)
-      throws IOException, InterruptedException{
+      throws IOException, InterruptedException {
     long startTime = System.currentTimeMillis();
     Socket socket;
     boolean connected = false;
     //See if port has opened for timeout.
-    while(System.currentTimeMillis() - startTime < timeout){
-      try{
+    while (System.currentTimeMillis() - startTime < timeout) {
+      try {
         socket = new Socket(host, port);
         socket.close();
         connected = true;
         break;
-      } catch (IOException e){
+      } catch (IOException e) {
         Thread.sleep(2000);
       }
     }
-    if(!connected) {
+    if (!connected) {
       throw new IOException("Port not opened within specified timeout.");
     }
   }
